
#ifndef CLUSTERInc
#define CLUSTERInc

#define MAX_INSTANCES            256
#define MAXPARAMSPERINSTANCE      16

#define MAX_GROUPS_PER_CLUSTER     8

#define MAX_CLUSTER_PARAMS       (MAX_GROUPS_PER_CLUSTER*MAX_INSTANCES*MAXPARAMSPERINSTANCE)
#define MAXPARAMSPERCLUSTER  MAX_CLUSTER_PARAMS

#define MAX_CLUSTER_ACTIONS        8
#define MAX_CLUSTER_HISTOGRAMS    MAX_CLUSTER_PARAMS

#define MAXLABELS               4096
#define MAXCHANNELS        MAXLABELS
#define MAXMODULES         (MAXLABELS/32)

#define MAX_CLUSTER_WIDTH     100000000

#include <rawdata.h>


/* Cluster Definition descriptor record ... */

#include <actions.h>

/* ParamRecord and PatternRecord must be same length */
/*  char param_labelname[NAMELEN];
 */

#define NORMAL_PARAM_TYPE     0
#define PATTERN_PARAM_TYPE    1

typedef struct param_record {
  char param_name[NAMELEN];
  int param_type;
  int label;
  int required;
  int histogram;
  int dummy;
} ParamRecord;

typedef struct pattern_record {
  char param_name[NAMELEN];
  int param_type;
  int group;
  int instance;
  int param;
  int pattern;
} PatternRecord;

typedef struct instance_record {
  int number;
  int nparams;
  int nrequired;
  int clusteroffset;
  ParamRecord param[MAXPARAMSPERINSTANCE];
} InstanceRecord;

typedef struct group_record {
  char instance_name[NAMELEN];
  int maxinstances;
  int ninstances;
  int ninstancesrequired;
  short eggroupbase;
  short outputmode;
  InstanceRecord instance[MAX_INSTANCES];
  int histlength1d;
  int histscale1d;
  int histlength2d;
  int histscale2d;
} GroupRecord;

typedef struct clusterdesc {
  char cluster_name[NAMELEN];
  char cluster_type[NAMELEN];
  char initialise_name[NAMELEN];
  char accept_name[NAMELEN];
  char viable_name[NAMELEN];
  char store_name[NAMELEN];
  int (*initialise)();
  int (*accept)();
  int (*viable)();
  int (*store)();
  void *dlhandle;
  int length;
  int width;
  int ngroups;
  int nhistograms;
  int nactions;
  int nfailactions;
  int fold;
  GroupRecord group[MAX_GROUPS_PER_CLUSTER];
  ActionRecord action[MAX_CLUSTER_ACTIONS];
  ActionRecord failaction[MAX_CLUSTER_ACTIONS];
  ActionRecord viableaction;
}ClusterDesc;

#endif


