#ifndef INCshmh
#define INCshmh

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <unistd.h>

#define MAX_SHM_PATH_LENGTH     127
#define MAX_SHM_NAME_LENGTH      31

#define MAX_SHM_SPECTRA        1024

#define MAX_SHM_SEGS            128

#ifdef SunOS
#define SHM_ATTACH_BASE  0xe0000000
#define MAX_SHM_SEG_SIZE (4*1024*1024)
#endif
#ifdef Linux
#define MAX_SHM_SEG_SIZE (32*1024*1024)      /* really (32*1024*1024) */
#endif

typedef struct SHM_SYSTEM_DESCRIPTION {
  int uid;
  int shmkey;
  int shmid;
  int usage;
  void *shmaddr;
  char  chars;
  char  directory[MAX_SHM_PATH_LENGTH];
} SHM_SYS_DESC;

int sysShmSize;         /* Size of basic shared memory table */
int sysShmid;           /* Shmid of basic shared memory table */
SHM_SYS_DESC *sysShmAddr;       /* Address of basic shared memory segment */

typedef struct SHM_HEADER_DESCRIPTION {
  int   maxShmSegSize;   /* Max allowed shared memory segment size */
  int   numShmSpec;   /* Number of shared memory spectra allocated */
  int   maxShmSpec;   /* Max number of shared memory spectra (system) */
  int spare1;
  int spare2;
  int   version;
} SHM_HDR_DESC;

SHM_HDR_DESC *shmHdrDesc;

typedef struct SHM_SEGMENT_DESCRIPTION {
  int   shmkey;
  unsigned int size;
#ifdef SIXTYFOURBIT
	char  *addr;
#else
	char  *addr;
        int   padding;
#endif
} SHM_SEG_DESC;

typedef struct SHM_SPECTRUM_DESCRIPTION {
  char  chars;
  char  name[MAX_SHM_NAME_LENGTH];
  int   shmdesc_index;
  unsigned int shmdesc_offset;
  unsigned int size;
} SHM_SPEC_DESC;

int userShmSize;         /* Size of basic shared memory table */

SHM_SEG_DESC *shmSegDesc;           /* Address of segment table */
SHM_SPEC_DESC *shmSpecDesc;           /* Address of spectrum table */

struct shmid_ds shmid_ds;          /* holds results of shmctl calls */

#endif /* INCshmh */
