loadsource common.unix
loadsource setupposn
loadsource setupgain

set mydebug 0

# Must exist one entry for each cluster template in config directory

set displayed(ngroups) 1
set displayed(group0_instance) 0
set zero_to_fifteen {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15}
set fold_values {1 2 3 4}

set default_histparams "16384 0 1024 0"

set env(tkmenubuttonactivebackground) aquamarine3
set env(tkmenubuttonactiveforeground) blue
set env(tkmenubuttonforeground) black
#set env(tkmenubuttonbackground) green
#set env(tkmenuactivebackground) blue
set env(tkmenuactiveforeground) WhiteSmoke
set env(tkmenubackground) WhiteSmoke
set env(tkmenuforeground) blue

set templates ""

# Cluster setup individual window routines

proc get-template-list {} {
    global templates
    set list ""
    foreach t $templates {lappend list [first $t]}
    return $list
}

proc read-templates {} {
    global basedir templates cluster_strings
    set ts [exec ls $basedir/config/templates]
    set templates ""
    foreach t $ts {
	if {![string is alnum $t]} {continue}

	set buf [read-files $basedir/config/templates/$t]
	set cl [first $buf]
	set n [string length [first $cl]]
	incr n
	set name [string range $cl $n end]
	lappend templates "$t $name"

	cluster-read-template $t
    }

    set cluster_strings ""
    foreach t $templates {
	lappend cluster_strings [first $t]
    }
}


proc cluster-from-type {type} {
    global clusterlist
    
    foreach clus $clusterlist {
	global $clus
	if {[set [set clus](type)] == $type} {return $clus}
    }
    
    return ""
}

proc cluster-from-name {name} {
    global clusterlist
    
    foreach clus $clusterlist {
	global $clus
	if {[set [set clus](name)] == $name} {return $clus}
	if {[string toupper [set [set clus](name)]] == $name} {return $clus}
	if {[string tolower [set [set clus](name)]] == $name} {return $clus}
    }
    
    return ""
}

proc cluster-name-from-name {name} {
    global clusterlist
    
    foreach clus $clusterlist {
	global $clus
	if {[set [set clus](name)] == $name} {return [set [set clus](name)]}
	if {[string toupper [set [set clus](name)]] == $name} {return [set [set clus](name)]}
	if {[string tolower [set [set clus](name)]] == $name} {return [set [set clus](name)]}
    }
    
    return ""
}

proc cluster-name-from-type {type} {
    global templates
    
    foreach t $templates {
	if {[second $t] == $type} {return [first $t]}
    }
    
    return ""
}

proc cluster-type-from-name {name} {
    global templates
    
    foreach t $templates {
	if {[first $t] == $name} {return [second $t]}
    }
    
    return ""
}

proc getnewclusternumber {} {
    global MAXCLUSTERS
    set nn 0
    while {$nn < $MAXCLUSTERS} {
	global clus$nn
	if {[array names clus$nn] == ""} {break}
	incr nn
    }
    
    return $nn
}

proc print_node_variables {t} {
    set trec [set t]
    global $trec
    
    set list ""
    set tn [array names $trec]
    foreach n $tn {lappend list  "[set trec]($n) [set [set trec]($n)]"}

    set acts [set [set trec](actions)]
    foreach a $acts {
	set tgrec [set t][set a]
	global $tgrec
	set tgn [array names $tgrec]
	puts ""
	foreach n $tgn {lappend list "[set tgrec]($n) [set [set tgrec]($n)]"}
    }
return $list
}

proc print_cluster_variables {t} {
    set trec [set t]
    global $trec [set trec]action
    
    set list ""
    set tn [array names $trec]
    foreach n $tn {lappend list "[set trec]($n) [set [set trec]($n)]"}
    set tn [array names [set trec]action]
    foreach n $tn {lappend list "[set trec]action($n) [set [set trec]action($n)]"}

    set ng [set [set trec](groups)]
    for {set g 0} {$g < $ng} {incr g} {
	set tgrec [set t]group[set g]
	global $tgrec
	set tgn [array names $tgrec]
	puts ""
	foreach n $tgn {lappend list "[set tgrec]($n) [set [set tgrec]($n)]"}
    }
return $list
}

proc print_template_variables {t} {
    set trec [set t]_template
    global $trec
    
    set list ""
    set tn [array names $trec]
    foreach n $tn {lappend list "[set trec]($n) [set [set trec]($n)]"}

    set ng [set [set trec](groups)]
    for {set g 0} {$g < $ng} {incr g} {
	set tgrec [set t]_template_group[set g]
	global $tgrec
	set tgn [array names $tgrec]
	puts ""
	foreach n $tgn {lappend list "[set tgrec]($n) [set [set tgrec]($n)]"}
    }
    return $list
}

proc cluster-set-variables {clustype} {
    global MAXCLUSTERS clusterlist default_histparams
    set tclus [set clustype]_template
    global $tclus
    
    set res ""
    foreach clus $clusterlist {
	global $clus
        set names [array names $clus]
	if {[lsearch $names type] != -1} {
	   if {[set [set clus](type)] == $clustype} {set res $clus; break;}
	} {	
	   set res $clus
	   break
	}
    }
    if {$res == ""} {
	set a [getnewclusternumber]
	if {$a == $MAXCLUSTERS} {
	    set-footer "Too many clusters defined"
	    return
	}
	set clus clus$a
	global $clus
    }

    set group 0
    set ngroups [set [set tclus](groups)]
    
    while {$group < $ngroups} {
	set tgname [set tclus]_group[set group]
	global $tgname
	set ninst [set [set tgname](maxinstances)]
	
	set grp [set [set tgname](name)]
	
	set gname [set clus]group[set group]
	global $gname

	set [set gname](name) $grp

	set inst 0
	while {$inst < $ninst} {
	    set n inst$inst
	    
	    set p 0
	    set nparams [set [set tgname](nparams)]
	    while {$p < $nparams} {
		set par [set [set tgname](param[set p])]

		if {[first $par] == "pattern"} {
		    set ps [first [second $par]]
		    set par [lreplace $par 1 1 $ps]
		    set [set gname](inst[set inst]param[set p]) $par
		} {
		    set [set gname](inst[set inst]param[set p]) $par
		}

		incr p
	    }
	    set [set gname](inst[set inst]nparams) $nparams
	    
	    incr inst
	}
	set [set gname](maxinstances) $ninst

	set histparams [set [set tgname](histparams)]

#	puts "[set clus] [set tgname] [set histparams]"

	set [set gname](histlen1d) [first $histparams]
	set [set gname](histscale1d) [second $histparams]
	set [set gname](histlen2d) [third $histparams]
	set [set gname](histscale2d) [fourth $histparams]

	incr group
    }

#    set [set clus](name) [cluster-name-from-type [set [set tclus](name)]]
#    set [set clus](type) [set [set tclus](name)]
    set [set clus](name) $clustype
    set [set clus](type) [set [set tclus](name)]

    set [set clus](groups) $ngroups
    set [set clus](maxgroups) $ngroups
    set [set clus](width) [set [set tclus](width)]
    set [set clus](accept) [first [set [set tclus](accept)]]
    set [set clus](init) [first [set [set tclus](init)]]
    set [set clus](store) [first [set [set tclus](store)]]
    set [set clus](viable) [set [set tclus](viable)] 
#    set [set clus](pileup) Veto
    set [set clus](actions) 0
    set [set clus](histparamlist) ""
    set [set clus](prefilter) [set [set tclus](prefilter)]
    set [set clus](bgocluster) [set [set tclus](bgocluster)]

    return $clus
}

proc cluster-read-template {clus} {
    global templatepath default_histparams
    
# Read template config file info ...
    
    set fullname $templatepath/$clus
    put-mydebug "Reading datafile $fullname..."
    set data [read-files $fullname]
    if {$data == ""} {
	puts stdout "returning -1"
	return
    }
    
    set trec [set clus]_template
    global $trec

    set [set trec](groups) 0
    set [set trec](width) 1
    set [set trec](accept) ""
    set [set trec](init) ""
    set [set trec](store) ""
    set [set trec](viable) "default fold 1"
    set [set trec](actions) 0
    set [set trec](xymatch) ""
    set [set trec](prefilter) ""
    set [set trec](bgocluster) false
    set [set trec](pileup) ""

    set param 0
    set inst 0
    set group -1
    set actions 0
    foreach l $data {
	set first [string range $l 0 0]
	if {$first != "#"} {
	    set keyword [lindex $l 0]
	    set ix [string first " " $l]
	    set dataval [string range $l [expr $ix + 1] end]
	    
	    put-mydebug "keyword=$keyword; dataval=$dataval"
	    
	    switch $keyword {
		cluster {
		    set inst 0
		    set group -1
		    set actions 0
		    set [set trec](name) [concat $dataval]

		}
		subsystem {

		    if {$param > 0} {
			for {set p 0} {$p < $param} {incr p} {
			    set par [split [set param[set p]]]
			    if {[first $par] == "pattern"} {
				set [set gname](param[set p]) "[set param[set p]]"
			    } {
				set [set gname](param[set p]) "[first $par] \"\" [string tolower [second $par]] [string tolower [third $par]] [string tolower [fourth $par]]"
			    }
			}
			set [set gname](nparams) $p
		    }

		    set param 0
		    
		    incr group
		    set gname [set trec]_group[set group]
		    
		    global $gname
		    set [set gname](maxinstances) [first $dataval]
		    set [set gname](histparams) $default_histparams
		    
		    if {[second $dataval] != -1} {
			set [set gname](name) [second $dataval]
		    } {
			set [set gname](name) ""
		    }
		    set ninst [first $dataval]
		    
		    incr [set trec](groups)
		}
		histogram {
		    set n [string length [first $l]]
		    set val [string range $l [expr $n +1] end]
		    set [set gname](histparams) $val
		}
		param {
		    set n [string length [first $l]]
		    set param[set param] [string range $l [expr $n +1] end]
		    incr param
		}
		pattern {
		    set n [string length [first $l]]
		    set param[set param] $l
		    incr param
		}
		action {
		    incr actions
		    set [set trec](action$actions) [concat $dataval]
		    set [set trec](actions) $actions
		}
		default {
		    set [set trec]($keyword) [concat $dataval]
		}
	    }
	}
    }
    
    if {$param > 0} {
	for {set p 0} {$p < $param} {incr p} {
	    set par [split [set param[set p]]]
	    if {[first $par] == "pattern"} {
		set [set gname](param[set p]) "[set param[set p]]"
	    } {
		set [set gname](param[set p]) "[first $par] \"\" [string tolower [second $par]] [string tolower [third $par]] [string tolower [fourth $par]]"
	    }
	}
	set [set gname](nparams) $p
    }
    
}

proc cluster-construct-panel {name} {
    global env iconimage np nc zero_to_fifteen fold_values
    global templates displayed
    global histranges histscales tacdelays

    set iconimage "@$env(ICONHOME)/midas"

    set clus [cluster-from-name $name]
    global $clus

    set cname [set name]
    set type [set [set clus](type)]

    image create bitmap Spectrum -file $env(ICONHOME)/spectrum \
        -maskfile $env(ICONHOME)/spectrum

    :frame [self frame-name].cluster :label [self frame-title] :show-footer T \
	:icon-name [self icon-title] :icon-image $iconimage :resize F
    [self frame-name].cluster configure -background wheat

    :panel $cname-PANEL :layout T :padx 2 :pady 2 :ipadx 1 :ipady 1 
# :row-gap 2
    :layout H
    set x [:path $cname-PANEL]
    $x configure -relief raised -bd 1

    set trec [set name]_template
    global $trec

    :message $cname-CLUSTER :label "Cluster type:   $name" :bold T
    #	   [:path $cname-CLUSTER].entry configure -relief flat
    :next-row T
	
    set ngroups [set [set trec](groups)]
    for {set group 0} {$group < $ngroups} {incr group} {

	:panel $cname-PANEL-$group :layout T :padx 2 :pady 2 :ipadx 1 :ipady 1
	# :row-gap 2 :col-gap 0
	:layout H
	set x [:path $cname-PANEL-$group]
	$x configure -relief raised -bd 1

	set tgrec [set trec]_group[set group]
	global $tgrec

	global [set clus]group[set group]
#	puts "[set trec] [set tgrec] [set [set clus]group[set group](maxinstances)]"

	set ninst [set [set tgrec](maxinstances)]
	if {$ninst > 1} {
	    set numchoice {}
	    for { set i 0 } { $i < $ninst } { incr i } {
		lappend numchoice $i
	    }
	    :choice-stack $cname-INSTANCE-$group :label Instance :exclusive T :strings $numchoice :value 0
	    set subname [set [set tgrec](name)]
	    :message $cname-INSTANCE-m$group :label "$subname" :bold T

	    :text $cname-NINSTANCE-$group :w 4 :value [set ninst] :label "      NInstances"
#	    pack $cname-NINSTANCE-$group.text -side right
	    :next-row T
	}

	set nparams [set [set tgrec](nparams)]
	for {set p 0} {$p < $nparams} {incr p} {
	    set par [set [set tgrec](param[set p])]
	    set pname [first $par]

	    if {$pname == "pattern"} {
		set ptype Pattern
		set pname [second $par]
		set pbit [fourth $par]
	    } {
		set ptype Param
		set pvxi [second $par]
		set sreq [string tolower [third $par]]
		set preq [string range $sreq end end]
		if {$preq == "y"} {set preq 1} {set preq 0}
		set shist [string tolower [fourth $par]]
		set phist [string range $shist end end]
		if {$phist == "y"} {set phist 1} {set phist 0}
	    }

	    if {$ptype == "Param"} {
		:message $cname-PARAMLABEL-$group-$p :label Param
		:message $cname-PARAM-$group-$p :label $pname :bold T :w 7
		:checkbox $cname-PARAM-$group-$p-REQ :label Req :value $preq
		:checkbox $cname-PARAM-$group-$p-HIST :label Hist :value $phist
		:button $cname-PARAM-$group-$p-SHOWHIST :label "Show" :w 3 :colour pink
		[:path $cname-PARAM-$group-$p-SHOWHIST] configure -image Spectrum -width 30 -height 18
		:text $cname-ADDRESS-$group-$p :label Adc :w 15 
		[:path $cname-ADDRESS-$group-$p].entry configure -bg white
		
		:menubutton $cname-ADDRESSHELP-$group-$p :label "Browse"
		bind [:path $cname-ADDRESSHELP-$group-$p] <1> "set-browsemenu $cname-ADDRESSHELP-$group-$p"
	    } {
		:message $cname-PATTERNLABEL-$group-$p :label Pattern
                set r [first $pname]
		:menubutton $cname-PATTERN-$group-$p-PARAMS :label "$r" :strings $pname
		:message $cname-PATTERNBITM-$group-$p :label Bit
		:menubutton $cname-PATTERN-$group-$p-BIT :label "$pbit" :strings $zero_to_fifteen
	    }

	    :next-row T
	}
    }


    :panel $cname-PANEL-C :layout T :padx 2 :pady 2 :ipadx 1 :ipady 1
    # :row-gap 2 :col-gap 0
    :layout H
    set x [:path $cname-PANEL-C]
    $x configure -relief raised -bd 1

    set val [set [set clus](width)]
    :slider $cname-WIDTH :label "Coinc width" :w 5 :value $val :valuex 80 :min 0 :max 1000 :resolution 1 :scale-length 10c
    [:path $cname-WIDTH].entry configure -bg white
    :next-row T
    
    if {[set [set trec](pileup)] != ""} {
	set s [set [set trec](pileup)]
      	set c [set [set clus](pileup)]
      	if {$c == ""} {set val 0} {set val [lsearch $s $c]}
	:choice-stack $cname-COND-PILEUP :label Pileup :exclusive T :strings $s :value $val
	:next-row T
    }
    if {[set [set trec](init)] != ""} {
	set s [set [set trec](init)]
      	set c [set [set clus](init)]
      	if {$c == ""} {set val 0} {set val [lsearch $s $c]}
	:choice-stack $cname-COND-INIT :label Init :exclusive T :strings $s :value $val
	:next-row T
    }
    if {[set [set trec](accept)] != ""} {
	set s [set [set trec](accept)]
      	set c [set [set clus](accept)]
      	if {$c == ""} {set val 0} {set val [lsearch $s $c]}
	:choice-stack $cname-COND-ACCEPT :label Accept :exclusive T :strings $s :value $val
	:next-row T
    }
    if {[set [set trec](viable)] != ""} {
	set s [set [set trec](viable)]
      	set c [set [set clus](viable)]
	set i [lsearch $s fold]
	if {$i >= 0} {
	    set s [lrange [set [set trec](viable)] 0 [expr $i -1]]
	} {
	    set s [set [set trec](viable)]
	}
	set i [lsearch $c fold]
	if {$i >= 0} {
	    set f [lindex [set [set clus](viable)] [expr $i +1]]
	} {
	    set f 1
	}
	if {$c == ""} {set val 0} {set val [lsearch $s [first $c]]}
#puts "$s...$c...$i...$f...$val"
	:choice-stack $cname-COND-VIABLE :label Viable :exclusive T :strings $s :value $val
	:choice-stack $cname-COND-FOLD :label Fold :exclusive T :strings $fold_values :value [lsearch $fold_values $f] :x 150
	:next-row T
    }
    if {[set [set trec](store)] != ""} {
	set s [set [set trec](store)]
      	set c [set [set clus](store)]
      	if {$c == ""} {set val 0} {set val [lsearch $s $c]}
	:choice-stack $cname-COND-STORE :label Store :exclusive T :strings $s :value $val
	:next-row T
    }
    
    # Add actions if any ...

    :panel $cname-PANEL-A :layout T :padx 2 :pady 2 :ipadx 1 :ipady 1
    # :row-gap 2 :col-gap 0
    :layout H
    set x [:path $cname-PANEL-A]
    $x configure -relief raised -bd 1

    set extras ""
    set nacts [set [set trec](actions)]
    for {set act 1} {$act <= $nacts} {incr act} {
	set extra [first [set [set trec](action$act)]]
	append extras " $extra"
    }
    set s "Histogram Tac $extras"
    :choice-stack $cname-ADDACTION :label "Add Action" :exclusive T :strings $s :value 0
   
    set nacts [set [set clus](actions)]

    global [set clus]action
    for {set a 0} {$a < $nacts} {incr a} {
	set action [set [set clus]action($a)]
	set acttype [first $action]
	set actname [second $action]

	set n [string length $acttype]
	set rest [string range $action [expr $n +1] end]
	set n [string length $actname]
	set rest [string range $rest [expr $n +1] end]

	set f $cname-ACTION-$a
	:panel $f :layout T :padx 2 :pady 2 :ipadx 1 :ipady 1
	# :row-gap 2 :col-gap 0
	:layout H
	[:path $f] configure -relief raised -bd 1

	:message $f-TYPE :w 15 :label $acttype
	:text $f-NAME :label "" :w 16 :value $actname
	[:path $f-NAME].entry configure -bg white

	:button $f-DELETE :label "Remove"
	:message $f-SPACEDELETE :label "    "

	switch $acttype {
	    histogram {
		set mode [first $rest]
		set xn [second $rest] 
		set xr [third $rest]
		set xs [fourth $rest]
		set yn [fifth $rest]
		set yr [sixth $rest]
		set ys [seventh $rest]

#Add spectrum mode

		:button $f-SHOWHIST :label "Show" :colour pink
		[:path $f-SHOWHIST] configure -image Spectrum -width 30 -height 18
		:checkbox $f-MD :label Indexed :value $mode

		:next-row T
		:message $f-XmN :w 15 :label "Xparam"
		:text $f-XN :label "" :w 16 :value $xn
		[:path $f-XN].entry configure -bg white
#		clus-drawhistentry $f-XN 15 $f $clus $a
		:choice-stack $f-XR :label "Range" :exclusive T :strings $histranges :value [lsearch $histranges $xr]
		:choice-stack $f-XS :label "Scale" :exclusive T :strings $histscales :value [lsearch $histscales $xs]

		:next-row T
		:message $f-YmN :w 15 :label "Yparam"
		:text $f-YN :label "" :w 16 :value $yn
		[:path $f-YN].entry configure -bg white
#		clus-drawhistentry $f-YN 15 $f $clus $a
		:choice-stack $f-YR :label "Range" :exclusive T :strings $histranges :value [lsearch $histranges $yr]
		:choice-stack $f-YS :label "Scale" :exclusive T :strings $histscales :value [lsearch $histscales $ys]

#		pack [:path $f-SHOWHIST] -side right

	    }
	    tac {
		set mode [first $rest]
		set xn [second $rest] 
		set yn [third $rest]
		set dl [fourth $rest]
		set g1 [fifth $rest]
		set g2 [sixth $rest]
		set xr [seventh $rest]
		set xs 0

#Add spectrum mode - show-menu?

		:button $f-SHOWHIST :label "Show" :colour pink
		[:path $f-SHOWHIST] configure -image Spectrum -width 30 -height 18
		:checkbox $f-MD :label Indexed :value $mode


		:next-row T
		set f $cname-ACTION-$a
		:message $f-XP :w 15 :label "Start"
		:text $f-XN :label "" :w 16 :value $xn
		[:path $f-XN].entry configure -bg white
#		clus-drawhistentry $f-XN 15 $f $clus $a
		:choice-stack $f-XR :label "Range" :exclusive T :strings $histranges :value [lsearch $histranges $xr]
		:choice-stack $f-XS :label "Scale" :exclusive T :strings $histscales :value [lsearch $histscales $xs]
		:next-row T
		:message $f-YP :w 15 :label "Stop"
		:text $f-YN :label "" :w 16 :value $yn
		[:path $f-YN].entry configure -bg white
#		clus-drawhistentry $f-YN 15 $f $clus $a
		:choice-stack $f-DL :label "Delay" :exclusive T :strings $histranges :value [lsearch $histranges $dl]
		:text $f-G1 :label "Gate" :w 5 :value $g1
		[:path $f-G1].entry configure -bg white
		:text $f-G2 :label "" :w 5 :value $g2
		[:path $f-G2].entry configure -bg white
	    }
	    default {
#puts "Cluster action $acttype chosen"
	    }
	}

    }

    # Initialise this instance ...
 
    for {set i 0} {$i < $ngroups} {incr i} {
	click-instance $cname-INSTANCE-$i 0
    }
    
    :show T
}

proc set-browsemenu {m} {
    global _winname menuoptions
    global vxicrate vxicratelist

    set mwid [:path $m].menu
    $mwid delete 0 end
    set ii 0
    set name [lowercase $m]
    set win $_winname($name)

    getvxiparams
    set boardlist [array names vxicrate]
    set boardlist [lsort -dictionary $boardlist]

    foreach c $boardlist {
	$mwid add cascade -label "$c" -menu $mwid.c$ii
	catch {eval menu $mwid.c$ii $menuoptions}
	:_set-font $mwid.c$ii menu 0 N
	$mwid.c$ii delete 0 end
	set jj 0
	
	foreach ss [set vxicrate([set c])] {
	    if {[expr $jj % 8] == 0} {set col 1} {set col 0}
	    $mwid.c$ii add command -label "$ss" -columnbreak $col \
		-command ":mymenubutton-event $win $name $c $ss"
	    incr jj
	}
	incr ii
    }
}

proc :mymenubutton-event {win name crate chan} {
    global frame _commandname

    set i [string first help $name 0]
    set wid [string toupper [string replace $name $i [expr $i + 4] -]]
    set x $crate.$chan
    :modify $wid :value $x
    
    click-addresshelp $name $crate $chan
}

proc click-add-action {wname v} {
    set l [split $wname "-"]
    set name [string toupper [first $l]]

    put-mydebug "click-add-action $wname $name $v"

    set name [cluster-name-from-name $name]
    set clus [cluster-from-name $name]

    global [set clus] [set clus]action

    set a [set [set clus](actions)]
    if {$a == ""} {set a 0}
    switch $v {
	Histogram {
	    set [set clus]action($a) {histogram UnNamed 0 X 16384 0 Y 16384 0}
	    incr a
	}
	Tac {
	    set [set clus]action($a) {tac UnNamed 0 X Y 8192 0 16384 16384}
	    incr a
	}
	default {
	    set trec [set name]_template
	    global $trec
	    for {set ta 1} {$ta <= [set [set trec](actions)]} {incr ta} {
#		puts "click-add-action: [set [set trec](action$ta)] ; $v"
		if {[first [set [set trec](action$ta)]] == $v} {
		    set [set clus]action($a) [set [set trec](action$ta)]
		    incr a
		}
	    }
	}
    }

   set [set clus](actions) $a
	
    cluster-construct-panel $name
}

proc click-remove-action {wname} {
    set l [split $wname "-"]
    set name [first $l]
    set a [third $l]

    put-mydebug "click-remove-action $wname $name $a"

    set name [cluster-name-from-name $name]
    set clus [cluster-from-name $name]

#    puts "click-remove-action $wname $a ... $name ... $clus"

    global $clus

    incr [set clus](actions) -1
    set nacts [set [set clus](actions)]

    global [set clus]action
    while {$a < $nacts} {
	set [set clus]action($a) [set [set clus]action([expr $a +1])]
	unset [set clus]action([expr $a +1])
	incr a
    }
	
    cluster-construct-panel $name
}

proc click-addresshelp {wname crate chan} {
    global displayed clusterparamlist vxiparamlist
    put-mydebug "click-addresshelp: $wname $crate $chan"
    set adc $crate.$chan

    set l [split $wname "-"]
    set type [first $l]
    set group [third $l]
    set param [fourth $l]

    set name [cluster-name-from-name $type]
    set clus [cluster-from-name $name]

    global $clus [set clus]group[set group]

    set inst [set displayed(group[set group]_instance)]
    set par [set [set clus]group[set group](inst[set inst]param[set param])]
    set pars [split $par " "]

    set newpar "[first $par] $adc [third $par] [fourth $par] [fifth $par]"
    set [set clus]group[set group](inst[set inst]param[set param]) $newpar

#Set new ADC channel entry in clusterparamlist...

    set p [cluster-set-param-name $clus $group $inst $par]
    set i [lsearch -regexp $clusterparamlist $p]
    set r [lindex $clusterparamlist $i]
    set r [lreplace $r 1 1 $adc]
    set clusterparamlist [lreplace $clusterparamlist $i $i $r]

#Set new entry in vxiparamlist...

    set i [lsearch -regexp $vxiparamlist $adc]
    set r [lindex $vxiparamlist $i]
    set r [lreplace $r 2 2 $p]
    set vxiparamlist [lreplace $vxiparamlist $i $i $r]
}

proc click-address {wname v} {
    global displayed vxiparamlist vxicrate
    put-mydebug "click-address: mname=$wname, v=$v"

    set l [split $wname "-"]
    set name [first $l]
    set group [third $l]
    set param [fourth $l]

    set name [cluster-name-from-name $name]
    set clus [cluster-from-name $name]
    global $clus [set clus]group[set group]

    set inst [set displayed(group[set group]_instance)]
    set par [set [set clus]group[set group](inst[set inst]param[set param])]
    set pars [split $par " "]
    
    set newpar "[first $par] $v [third $par] [fourth $par] [fifth $par]"
    set [set clus]group[set group](inst[set inst]param[set param]) $newpar

#Set new entry in vxiparamlist...
    set adc "[set name]_S[set inst].[first $par]"
    set i [lsearch -regexp $vxiparamlist $adc]
    set r [lindex $vxiparamlist $i]
    set r [lreplace $r 0 0 $v]
    set vxiparamlist [lreplace $vxiparamlist $i $i $r]
#Set new entry in vxicrate ...
    # Extract crate name ...
    set n [string first "." $v]
    incr n -1
    set crate [string range $v 0 $n]
    addvxicrate $crate
    set chans ""
    for {set chan 0} {$chan < 32} {incr chan} {append chans " c$chan" }
    set vxicrate($crate) $chans

}

proc click-param {wname v} {
    global displayed
    put-mydebug "click-param: $wname $v"

    set l [split $wname "-"]
    set name [first $l]
    set group [third $l]
    set param [fourth $l]
    set opt   [fifth $l]

    set name [cluster-name-from-name $name]
    set clus [cluster-from-name $name]
    global $clus [set clus]group[set group]

    set inst [set displayed(group[set group]_instance)]
    set par [set [set clus]group[set group](inst[set inst]param[set param])]

    if {$v == 1} {set y y} {set y n}
    switch $opt {
	REQ {
	    set newpar "[first $par] [second $par] req=$y [fourth $par] [fifth $par]"
	    set [set clus]group[set group](inst[set inst]param[set param]) $newpar
	}
	HIST {
	    set newpar "[first $par] [second $par] [third $par] hist=$y [fifth $par]"
	    set [set clus]group[set group](inst[set inst]param[set param]) $newpar

	    if {!$v} {
		:modify [first $l]-PARAM-$group-$param-SHOWHIST :inactive T
	    } {
		:modify [first $l]-PARAM-$group-$param-SHOWHIST :inactive F
	    }
	}
	SHOWHIST {
	    click-show-param-histogram $clus $group $inst $par
	}

    }
}

proc click-pattern {wname v} {
    global displayed
    put-mydebug "click-pattern: $wname $v"

    set l [split $wname "-"]
    set name [first $l]
    set group [third $l]
    set param [fourth $l]
    set opt   [fifth $l]

    set name [cluster-name-from-name $name]
    set clus [cluster-from-name $name]
    global $clus [set clus]group[set group]

    set inst [set displayed(group[set group]_instance)]
    set par [set [set clus]group[set group](inst[set inst]param[set param])]

    switch $opt {
	PARAMS {
	    if {$v == ""} {set $v "PAT1"}
	    set newpar "pattern $v bit [fourth $par]"
	    set [set clus]group[set group](inst[set inst]param[set param]) $newpar
            :modify [first $l]-PATTERN-$group-$param-PARAMS :label $v
	}
	BIT {
	    set newpar "pattern [second $par] bit $v"
	    set [set clus]group[set group](inst[set inst]param[set param]) $newpar
            :modify [first $l]-PATTERN-$group-$param-BIT :label $v
	}

    }
}

proc click-width {wname v} {
    put-mydebug "click-width: $wname $v"

    set l [split $wname "-"]
    set type [first $l]

    set name [cluster-name-from-name $type]
    set clus [cluster-from-name $name]
    global $clus

    set [set clus](width) $v
}

proc click-ninstances {wname v} {
    put-mydebug "click-ninstances: $wname $v"

    set l [split $wname "-"]
    set name [first $l]
    set group [third $l]
    set param [fourth $l]

    set name [cluster-name-from-name $name]
    set clus [cluster-from-name $name]
    global $clus [set clus]group[set group]

    set trec [set name]_template
    set tgrec [set trec]_group[set group]
    global $tgrec

    set [set tgrec](maxinstances) $v


# Set extra instance records

    set ninst [set [set clus]group[set group](maxinstances)]
    while {$ninst < $v} {
	set npars [set [set clus]group[set group](inst[expr $ninst -1]nparams)]
	set [set clus]group[set group](inst[set ninst]nparams) $npars
	for {set p 0} {$p < $npars} {incr p} {
	    set par [set clus]group[set group](inst[expr $ninst -1]param[set p])
	    set [set clus]group[set group](inst[set ninst]param[set p]) $par
	}
	incr ninst
    }

    set [set clus]group[set group](maxinstances) $v
}

proc click-instance {wname v} {
    global displayed
    put-mydebug "click-instance: $wname $v"

    set l [split $wname "-"]
    set name [first $l]
    set group [third $l]

    set clus [cluster-from-name $name]

#    puts "click-instance: $wname $v ... $name...$clus"

    global $clus [set clus]group[set group]

    set np [set [set clus]group[set group](inst[set v]nparams)]
    for {set p 0} {$p < $np} {incr p} {
	set par [set [set clus]group[set group](inst[set v]param[set p])]

	if {[first $par] != "pattern"} {
	    set sreq [string tolower [third $par]]
	    set preq [string range $sreq end end]
	    if {$preq == "y"} {set preq 1} {set preq 0}
	    set shist [string tolower [fourth $par]]
	    set phist [string range $shist end end]
	    if {$phist == "y"} {set phist 1} {set phist 0}
	    set pvxi [second $par]
	    
	    :modify [first $l]-PARAM-[set group]-[set p]-REQ :value $preq
	    :modify [first $l]-PARAM-[set group]-[set p]-HIST :value $phist
	    :modify [first $l]-ADDRESS-[set group]-[set p] :value $pvxi
	    
	    if {!$phist} {
		:modify [first $l]-PARAM-$group-$p-SHOWHIST :inactive T
	    } {
		:modify [first $l]-PARAM-$group-$p-SHOWHIST :inactive F
	    }
	} {
            :modify [first $l]-PATTERN-$group-$p-PARAMS :label [second $par]
            :modify [first $l]-PATTERN-$group-$p-BIT :label [fourth $par]
	}

    }
    
    set displayed(group[set group]_instance) $v
}

proc click-cluster-list {v i} {
    global cluster-list-selection
    set cluster-list-selection $i
}

proc click-condition {wname v} {
    set l [split $wname "-"]
    set name [first $l]

    set name [cluster-name-from-name $name]
    set cond [string tolower [third $l]]

    put-mydebug "click-condition: $wname $v ... $name $cond"

    set clus [cluster-from-name $name]
    global $clus

    switch $cond {
	fold {
	    set i [llength [set [set clus](viable)]]
	    incr i -1
	    set [set clus](viable) [lreplace [set [set clus](viable)] $i $i $v]
	}
	viable {
	    set [set clus](viable) [lreplace [set [set clus](viable)] 0 0 $v]
	}
	default 
	{
	    set [set clus]($cond) $v
	}
    }
}

proc cluster-set-param-name {clus group inst paramrecord} {
    global $clus [set clus]group[set group]
    set cname [set [set clus](name)]
    set gname [set [set clus]group[set group](name)]
    set maxinst [set [set clus]group[set group](maxinstances)]
    set pname [first $paramrecord]
    if {$gname != ""} {set gname "_[set gname]"}
    if {$maxinst > 1} {set iname ".$inst"} {set iname ""}
    set f [set cname][set gname][set iname].[set pname]
    put-mydebug  "[set cname] [set gname] [set iname] [set pname]... $f"
    return $f
}

proc click-show-param-histogram {clus group inst param} {
    global specbase $clus [set clus]group[set group] tdrEBResource
    set f [cluster-set-param-name $clus $group $inst $param]
    set sortresource /$tdrEBResource/
    inform sd show $sortresource$specbase/$f
}

proc click-show-action-histogram {clus act} {
    global specbase $clus [set clus]action tdrEBResource
    set cname [set [set clus](name)]
    set action [set [set clus]action($act)]
    set pname [second $action]
    set f [set cname].[set pname]
    set sortresource /$tdrEBResource/
    inform sd show $sortresource$specbase/$f
}

proc click-modify-action {wname val v} {
    set l [split $wname "-"]
    set name [first $l]
    set a [third $l]
    set fn [fourth $l]

    set name [cluster-name-from-name $name]
    set clus [cluster-from-name $name]

#    puts "click-modify-action: $wname $v ... $name...$clus"

    global $clus [set clus]action

    set action [set [set clus]action($a)]
    set act [first $action]
    switch $act {
	histogram {
	    switch $fn {
		SHOWHIST {click-show-action-histogram $clus $a;return}
		DELETE {click-remove-action $wname;return}
		NAME {set action [lreplace $action 1 1 $val]}
		MD {set action [lreplace $action 2 2 $val]}
		XN {set action [lreplace $action 3 3 $val]}
		XR {set action [lreplace $action 4 4 $v]}
		XS {set action [lreplace $action 5 5 $v]}
		YN {set action [lreplace $action 6 6 $val]}
		YR {set action [lreplace $action 7 7  $v]}
		YS {set action [lreplace $action 8 8 $v]}
	    }
	}
	tac {
	    switch $fn {
		SHOWHIST {click-show-action-histogram $clus $a;return}
		DELETE {click-remove-action $wname;return}
		NAME {set action [lreplace $action 1 1 $val]}
		MD {set action [lreplace $action 2 2 $val]}
		XN {set action [lreplace $action 3 3 $val]}
		YN {set action [lreplace $action 4 4 $val]}
		DL {set action [lreplace $action 5 5 $v]}
		G1 {set action [lreplace $action 6 6 $v]}
		G2 {set action [lreplace $action 7 7 $v]}
		XR {set action [lreplace $action 8 8 $v]}
		XS {}
	    }
	}
	default {
	    switch $fn {
		SHOWHIST {click-show-action-histogram $clus $a;return}
		DELETE {click-remove-action $wname;return}
		NAME {set action [lreplace $action 1 1 $val]}
	    }
	}
    }
    set [set clus]action($a) $action
}

proc click {arg} {
    global debug WIDGET COMMAND FRAME VALUE ITEM
    if {$debug} then {insert-debug "click with argument \{$arg\}"}
    set f [lindex $arg 0]
    set FRAME $f
    set c [lindex $arg 1]
    set COMMAND $c
    set w [lindex $arg 2]
    set WIDGET $w
    set v [lindex $arg 3]
    set VALUE $v
    set i [lindex $arg 4]
    set ITEM $i
    catch {set i2 [lindex $arg 5]}
    if {[info exists i2]} {set ITEM2 $i2}
    if {$c==":quit"} then {return [click-quit $f]}

    switch -glob $w {
        *ADDRESSHELP*  {click-addresshelp $w $i $i2}
        *ADDRESS-*     {click-address $w $v}
        *NINSTANCE-*    {click-ninstances $w $v}
        *INSTANCE-*    {click-instance $w $v}
        *PARAM-*       {click-param $w $v}
        *-PATTERN-*     {click-pattern $w $v}
        *COND-*        {click-condition $w $i}
	*ADDACTION*    {click-add-action $w $i}
	*-ACTION-*     {click-modify-action $w $v $i}
	*-WIDTH*       {click-width $w $v}
        default        {
	    set p "click-[lowercase $w]"
	    if {[info procs $p] == "$p"} then {
		set argnum [llength [info args $p]]
		if {$argnum == 0} then {$p}
		if {$argnum == 1} then {$p "$v"}
		if {$argnum == 2} then {$p "$v" "$i"}
		if {$argnum > 2} then {insert-debug "click: $p expects too many args ($argnum)"}
	    } else {
		set p "click-default"
		if {[info procs $p] == "$p"} then {
		    set argnum [llength [info args $p]]
		    if {$argnum == 0} then {$p}
		    if {$argnum == 1} then {$p $w}
		    if {$argnum == 2} then {$p $w "$v"}
		    if {$argnum == 3} then {$p $w "$v" "$i"}
		    if {$argnum > 3} then {insert-debug "click: $p expects too many args ($argnum)"}
		} else {
		    insert-debug "click with argument \{$arg\}"
		}
	    }
	}
    }
}


# Cluster setup base window routines
set initing_window 0

proc init-window {f} {
    global env framename frametitle icontitle iconimage initing_window
    global templates cluster_strings cluster_names cluster_types clusterlist

    if {$initing_window} {return}
    set initing_window 1

    set framename [self frame-name]
    set frametitle "$f Setup Frame"
    set icontitle $f
    set iconimage "@$env(ICONHOME)/midas"

    :frame [self frame-name].clusters :label "[self frame-title]" :show-footer T \
	:icon-name "[self icon-title]"  :icon-image $iconimage :resize T

    :container LEFT :layout T :anchor W :relief R :pady 3

    :panel L1 :layout H :row-gap 10 :col-gap 10
    :layout V
    :message LIST-HEADER  :label "Clusters" :bold T :x 40

    set cluster_names {}
    set cluster_types {}
    foreach clus $clusterlist {
	global $clus
	set type [set [set clus](type)]
	lappend cluster_types $type
	lappend cluster_names "[set [set clus](name)]"
    }

    :list CLUSTER-LIST :rows 12  :list-width 140 :x 0 :exclusive T \
	     :double-click T :strings $cluster_names
    bind [:path CLUSTER-LIST] <3> "show-cluster-variables [:path CLUSTER-LIST]"

    :panel L2 :layout H :row-gap 10 :col-gap 10

    read-templates

    :menubutton ADD-CLUSTER :label "Add Cluster" :strings $cluster_strings \
	:command click-add-cluster :y 35

    :next-row T
    :button  DELETE :label "Remove" :command click-remove-cluster
    :next-row T
    :next-row T
    :button  SHOWPOSN :label "Posn Matching" :command click-show-posmatch
    :next-row T
    :button  SHOWGAIN :label "Gain Matching" :command click-show-gainmatch
    :next-row T
    :button  SHOWVARS :label "Variables" :command display-cluster-variables 
    :next-row T
    :button  SHOWTEMP :label "Template" :command display-template-variables 

   pack [:path L2] -after [:path L1] -side top

    :show T
    set initing_window 0
 }

proc click-show-posmatch {} {
    global clusterlist
    clear-footer

    set win [:path CLUSTER-LIST]
    set i [$win.list curselection]
    if {$i == ""} {set-footer "No selection";return}
    set clus [lindex $clusterlist $i]
    global $clus
    set t [set [set clus](name)]
    draw-posn-frame $t
}

proc click-show-gainmatch {} {
    global clusterlist
    clear-footer

    set win [:path CLUSTER-LIST]
    set i [$win.list curselection]
    if {$i == ""} {set-footer "No selection";return}
    set clus [lindex $clusterlist $i]
    global $clus
    set t [set [set clus](name)]
    draw-gain-frame $t
}

proc display-cluster-variables {} {
    show-cluster-variables [:path CLUSTER-LIST]
}

proc display-template-variables {} {
    global clusterlist
    clear-footer
    set win [:path CLUSTER-LIST]
    set i [$win.list curselection]
    if {$i == ""} {set-footer "No selection";return}
    set clus [lindex $clusterlist $i]
    global $clus
    set t [set [set clus](name)]
    show-template-variables $t
}

proc put-mydebug {s} {
    global mydebug
    if {$mydebug} {puts stdout $s}
}

proc click-remove-cluster {} {
    global cluster_names cluster_types cluster-list-selection clusterlist

    set cindex [set cluster-list-selection]
    set name [lindex cluster_names $cindex]
    set cluster_names [lreplace $cluster_names $cindex $cindex]

    set type [lindex cluster_types $cindex]
    set cluster_types [lreplace $cluster_types $cindex $cindex]

    set clus [lindex clusterlist $cindex]
    zap-cluster $clus
    set clusterlist [lreplace $clusterlist $cindex $cindex]

    set sel [llength $cluster_names]
    :modify CLUSTER-LIST :empty T :strings $cluster_names :select $sel :see $sel
}

proc click-add-cluster {p m cindex} {
    global templates cluster_names cluster-list-selection
    set t [lindex $templates $cindex]
    set name [first $t]
    set type [second $t]

#puts "click-add-cluster: $name $type"

    if {[lsearch -exact $cluster_names $name] == -1} {
	set cluster-list-selection $cindex
	
	add-cluster $name

	set sel [llength $cluster_names]
	:modify CLUSTER-LIST :empty T :strings $cluster_names :select $sel :see $sel

	cluster-draw-window name $name
    } {
	set-footer "Detector type already in list ..."
    }
}

proc add-cluster {name} {
    global templates cluster_names cluster_types clusterlist

    foreach t $templates {
	if {[first $t] == $name} {set type [second $t]}
	}
    if {$type == ""} {return}

    cluster-read-template $name
    set clus [cluster-set-variables $name]
    lappend cluster_types $type
    lappend cluster_names $name
    
    append clusterlist " $clus"
}

proc cluster-draw-window {type name} {
    if {$type == "node"} {
	global $name
	set tname [set [set name](detectors)]
	if {$tname == ""} {set-footer "Detector not defined"; return}
    } {
	set tname $name
    }

    cluster-construct-panel $tname
}

proc double-click-cluster-list {name index} {
    global clusterlist

    foreach clus $clusterlist {
	global $clus
	set n [set [set clus](name)]

	if {$n == $name} {
	    cluster-draw-window type $name
	}
    }
}

proc double-click {arg} {
   global debug WIDGET COMMAND FRAME VALUE ITEM
    put-mydebug "double-click: $arg"
   set f [lindex $arg 0]
   set FRAME $f
   set c [lindex $arg 1]
   set COMMAND $c
   set w [lindex $arg 2]
   set WIDGET $w
   set v [lindex $arg 3]
   set VALUE $v
   set i [lindex $arg 4]
   set ITEM $i
   set p "double-click-[lowercase $w]"

   if {[info procs $p] == "$p"} then {
      set argnum [llength [info args $p]]
      if {$argnum == 0} then {$p}
      if {$argnum == 1} then {$p "$v"}
      if {$argnum == 2} then {$p "$v" "$i"}
      if {$argnum > 2} then {insert-debug "double-click: $p expects too many args ($argnum)"}
   } else {
      set p "double-click-default"
      if {[info procs $p] == "$p"} then {
         set argnum [llength [info args $p]]
         if {$argnum == 0} then {$p}
         if {$argnum == 1} then {$p $w}
         if {$argnum == 2} then {$p $w "$v"}
         if {$argnum == 3} then {$p $w "$v" "$i"}
         if {$argnum > 3} then {insert-debug "double-click: $p expects too many args ($argnum)"}
      } else {
         insert-debug "double-click with argument \{$arg\}"
      }
   }
}

proc click-cluster-setup {} {
    global wintitle
    
    set wintitle "Great Dacq"
    init-window $wintitle
	
}

