set p [package require SOAP]
puts "SOAP $p"
set resource ""
set preferences(server-host) localhost
set preferences(server-port) 15560
load_preferences
set host $preferences(server-host)
set port $preferences(server-port)

proc click-host {v} {
    global resource host preferences soap_server
    set host $v
    set preferences(server-host) $host
    update_preferences
    server-connect $soap_server
puts "click-host $v"
}

proc click-port {v} {
    global resource host port preferences soap_server
    set port $v
    set preferences(server-port) $port
    update_preferences
    server-connect $soap_server
}

proc server-connect {server} {
    global host port

#    set host [inform $server set host]
#    set port [inform $server set port]
#puts "$server: host $host port $port"
    variable uri "urn:$server"
    variable proxy "http://[set host]:[set port]/"
    
    SOAP::create ping -proxy $proxy -uri $uri -params {}
    SOAP::create command -proxy $proxy -uri $uri -params {classname string commandname string value string}

    set z [server_ping $server]
    return $z
}
    
proc soap-error {m} {
    if {[string range $m 0 19] == "SOAP transport error"} {set-footer "SOAP transport error"}
    if {[string range $m 0 8] == "SOAP-ENV:"} {set-footer "[string range $m 9 end]"}
}

proc server_ping {server} {
    set m ""
    set z [catch {set s [ping]} m ]
    if {!$z} {return 1} {soap-error $m; return 0}
}

proc server_command {classname commandname value} {
set m ""
set z [catch {set s [command $classname $commandname $value]} m ]
if {!$z} {return $m} {soap-error $m; return $value}
}

# =============================================================

proc fopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	display-error-report self $m
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
	if { ! [isComment? $t] } { lappend buf $t }
    }
    close $fp
    return $buf
}

proc write-file {f buf} {
    set fp [fopen $f w+]
    foreach line $buf {
	puts $fp $line
    }
    close $fp
    set-footer "Saved to $f"
}

# Process line pairsa to generate register definitions ..
#e.g. ersdefine state state
#     erswta -a STATE

proc define_registers {register_file} {
    global register_list

    set regfile [read-file $register_file]

    array unset register_list *
    
    foreach l $regfile {
	switch [first $l] {
	    ersdefine {
		set local_name [second $l]
		set class [third $l]
	    }
	    erswta {
		set reg_name [string tolower [third $l]]
		set index [fourth $l]
		if {$index != ""} {append reg_name $index}
		set register_list($local_name) "$class \"$reg_name\""
	    }
	    default {
	    }
	}
    }
}

proc readreg {name} {
    global register_list
    
    set class [first $register_list($name)]
    set reg [second $register_list($name)]

    set value [server_command $class $reg "?"]
    return $value
}

proc readregs {name} {
    global register_list
    
    set ll [array names register_list $name]
    set value ""
    foreach l $ll {
	lappend value "$l [readreg $l]"
    }
    return $value
}

proc writereg {name value} {
    global register_list
    
    set class [first $register_list($name)]
    set reg [second $register_list($name)]

    set ret [server_command $class $reg $value]
    return $ret
}

proc writeregs {name value} {
    global register_list
    
    set ll [array names register_list $name]
    foreach l $ll {
	set class [first $register_list($l)]
	set reg [second $register_list($l)]
	
	set ret [server_command $class $reg $value]
	if {$ret} {break }
    }
    return $ret
}

