set vxiparamlist ""
set vxicratelist ""

set sortbystrings "ADCname ADClabel ParamName"
set sortby 0

set params_selection 0

set nullparamname "z_"

set maxADCcards 64
set maxPATTERNcards 8

proc addvxicrate {crate} {
    global vxicratelist vxicrate
    foreach c $vxicratelist {
	if {$c == $crate} {return}
    }
    append vxicratelist " $crate"
    set vxicrate($crate) ""
}

proc initvxiparams {} {
    global vxicrate vxiparamlist vxicratelist clusterparamlist nullparamname
    
    set v [array names vxicrate]
    foreach c $v { unset vxicrate($c) }

    set vxiparamlist ""
    set vxicratelist ""

    set param $nullparamname
    set adjust 0

    foreach c $clusterparamlist {
	
	set param [first $c]
	
	set pars [split [third $c] "("]
	global [first $pars]
	
	set par [set [third $c]]
	set dts [fifth $par]
	set i [string first "=" $dts ]
	if {$i != -1} {
	    incr i
	    set adjust [string range $dts $i end]
	} {
	    set adjust 0
	}

	set name [second $c]

	# Extract crate name ...
	set n [string first "." $name]
	incr n -1
	set crate [string range $name 0 $n]
	set board [string range $name 3 $n]
	incr n 2
	set chan [string range $name $n end]
	addvxicrate $crate
	append vxicrate($crate) " $chan"


	set label [expr $board * 32 + [string range $chan 1 end]]
	lappend vxiparamlist "[format "%-16s %5s %6d %24s" $name $label $adjust $param]"
    }

    getvxiparams
}

proc getvxiparams {} {
    global vxicrate vxiparamlist vxicratelist

    set chans ""
    for {set chan 0} {$chan < 32} {incr chan} {
	append chans " c$chan"
    }
    set v [array names vxicrate]
    foreach c $v {
	set vxicrate($c) $chans
    }

}

proc click-sortby {p m v} {
    global sortby sortbystrings
    clear-footer

    set sortby $v

    :modify SORTBY :label "Sort By ... [lindex $sortbystrings $v]"

   paint-vxilist
}

set initing_window 0

proc MakeParamsFrame {} {
    global vxiparamlist sortby sortbystrings params_selection initing_window

    if {$initing_window} {return}
    set initing_window 1

    :frame [self frame-name].setup :label "TDR Event Builder Parameter Mappings" :show-footer T \
	:icon-name "[self icon-title]" :resize F
    
    :panel LISTING  :layout R :padx 10 :pady 2
      :layout H; :row-gap 10; :col-gap 5
      :message PHEADER :x 00 :w 70 \
	:label "ADC-names           ADC-labels   TS-adjust                        Param-Names"
      :next-row T
      :list VXILIST  :rows 32 :list-width 500 :x 0 :exclusive T \
	:double-click F :command click-vxilist :see 0
      [:path VXILIST].list configure -background white

      :next-row T
    set val 0
    :slider ADJUST :label "TSadjust" :value $val :min -25 :max 25 :resolution 1
    [:path ADJUST].entry configure -bg white
    :message CURSEL :label "Current [first [lindex $vxiparamlist $params_selection]]"

      :next-row T
      :button REFRESH :label "Refresh" :command click-vxirefresh
      :menubutton SORTBY :w 20 :label "Sort By ... [lindex $sortbystrings $sortby] " \
	:command click-sortby :strings $sortbystrings

      :button MYHELP  :label "Help" :command click-help

    :show T
    set initing_window 0
}

proc click-vxilist {c f i} {
    global vxiparamlist params_selection
    set vc [first $f]
    set params_selection [lsearch -regexp $vxiparamlist $vc]
    :modify CURSEL :label "Current [first [lindex $vxiparamlist $params_selection]]"
}

proc click-adjust {v} {
    global vxiparamlist clusterparamlist params_selection nullparamname
    set i $params_selection
    set r [lindex $vxiparamlist $i]
    set r [lreplace $r 2 2 $v]
    set vxiparamlist [lreplace $vxiparamlist $i $i $r]

    if {[fourth $r] != $nullparamname} {
	set name [first $r]
	foreach c $clusterparamlist {
	    if {[second $c] == $name} {
		set pars [split [third $c] "("]
		global [first $pars]
		
		set par [set [third $c]]
		puts "click-adjust: $par"
		set par [lreplace $par 4 4 "dts=$v"]
		set [third $c] $par
		break
	    }
	}
	puts "click-adjust: $par ... [set [third $c]]"
	
	paint-vxilist
    }
}

proc paint-vxilist {} {
    global debug vxiparamlist sortby params_selection
    if {$debug} {puts stderr "paint-vxilist: "}
   
    switch $sortby {
	0 {set newlist [lsort -dictionary -index $sortby $vxiparamlist]}
	1 {set newlist [lsort -integer -index $sortby $vxiparamlist]}
	2 {set newlist [lsort -dictionary -index 3 $vxiparamlist]}
    }
    set drawlist ""
    foreach l $newlist {
	set p [fourth $l]
	if {$p == "z_"} {set p "_"}
	set l [format "%-16s %5s %6d %24s" [first $l] [second $l] [third $l] $p]
	lappend drawlist $l
    }
    :modify VXILIST :empty T :strings $drawlist :see $params_selection
}

proc click-vxirefresh {} {
   global debug
   if {$debug} then {puts stderr "click-vxirefresh"}
   getvxiparams
   paint-vxilist
   }

proc draw-params-frame {} {
    getvxiparams
    MakeParamsFrame 
    paint-vxilist
}
