
set frametitle "TDR Output Disc Handler Control"

set currentvolume "DISC"

set z [catch {set env(OSTYPE)} m]
if {$z} {set ostype solaris} {set ostype $m}
if {$ostype == "solaris"} {
  set devlist {"Not_Active" "Disc"}
  } {
  set devlist {"Not_Active" "Disc"}
  }

#------------------------------------------------------------------------------

proc click arg {
   global debug chan
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      ADD                    {click-add}
      INSERT                 {click-insert}
      REPLACE                {click-replace}
      REMOVE                 {click-remove}
      RESETLINE              {click-resetline}
      SAVE                   {click-save}

      MYHELP                   {click-help}
      REFRESH                {click-refresh}

      RUNLIST                {click-runlist $c $v}

      READRUNFILES           {click-load-runfiles}

      FILE                   {click-file $v}
      STARTBLOCK             {click-startblock $v}
      ENDBLOCK               {click-endblock $v}
      MAXFILE                {click-maxfile $v}


      default                {puts stdout $arg}
   }
}
#------------------------------------------------------------------------------

proc make-frame {} {
   global debug env outputlabel outchannel maxfilesize
   global handlerlist handlerselected handlerloaded status
   global devlist debugmenustrings runfilelist
   global currentstart currentend

   if {$debug} {puts stdout "make-frame: frame=[self frame-name]"}
   if {$debug} {puts stdout "...devlist=$devlist"}

  set vx1 12c
  set vx2 19c
  set vx3 200
  set vx4 380
  set twid1 15
  set twid2 45

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/sort" \
      :icon-name "[self icon-title]" :resize F

  :panel BUTTONS  :layout B :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 5
      :button ADD :label "Add" :w 2
      :button INSERT :label "Insert" :w 2
      :button REPLACE :label "Replace" :w 5
      :button REMOVE :label "Remove" :w 5
      :button RESETLINE :label "ResetLine" :w 6
      :button SAVE :label "Save" :w 4
      :button REFRESH :label "Refresh" :w 8 :x $vx4 
      :button MYHELP :label "Help" :w 7
   :panel RUNFILES :layout B :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20
      :message LIST-HEADER :label "Output Runfiles" :bold T :x 280
      :list RUNLIST :rows 10 :list-width 550 :x 0 :exclusive T \
             :strings $runfilelist :see 0
     :next-row T
      :text FILE :label "File:" :w $twid2 :valuex $vx1
     :next-row T
      :text STARTBLOCK :label "Start block:" :value $currentstart :valuex $vx1
      :text MAXFILE :label "MAX (Gbytes):" :w 3 :valuex $vx3
     :next-row T
      :text ENDBLOCK :label "End block:" :value $currentend :valuex $vx1
    :button READRUNFILES :label "Load from outruninfo[set outchannel] File" :x $vx4
  :panel OPTIONS  :layout L :padx 10 :pady 10
    :layout H; :row-gap 20; :col-gap 10
    :message OUTPUT-HANDLER :label "Output handler:    $handlerloaded" :bold T
    :message OUTPUT-TYPE :w 20 :x 200 :label "[set outputlabel($outchannel)]" :bold T
    :text OUTPUT-HANDLER-STATUS :label "Status:" :w 10 :value $status :bold T \
       :x $vx4
    [:path OUTPUT-HANDLER-STATUS].entry configure -bd 0
    pack [:path OUTPUT-HANDLER-STATUS] -side right
    :show T
}

proc paint-handler-status {} {
   global status basedir
   set status 0
   :modify OUTPUT-HANDLER-STATUS :value $status
   }

proc repaint {} {
   global handlerloaded outputlabel outchannel

    :modify READRUNFILES :label "Load from outruninfo[set outchannel] File"
   :modify OUTPUT-HANDLER :label "Output handler:    $handlerloaded"
   :modify OUTPUT-TYPE :label "[set outputlabel($outchannel)]"
   paint-handler-status

   paint-runfiles
   }

proc paint-runfiles {} {
   global runfilelist runfileselected maxfilesize
   global currentvolume currentfile currentstart currentend

   set i 0
   set list ""
   foreach x $runfilelist {
       lappend list [format-runfile "$i [second $x] [third $x] [fourth $x] [fifth $x]"]
       incr i
   }
   set runfilelist $list

   :modify RUNLIST :empty T :strings $runfilelist :select $runfileselected :see $runfileselected 
    if {$runfilelist != {} } {
   set runfile [lindex $runfilelist $runfileselected]
   set runfileselected [first $runfile]
   set currentvolume [second $runfile]
   set currentfile [third $runfile]
   set currentstart [fourth $runfile]
   set currentend [fifth $runfile]
   :modify FILE :value $currentfile
   :modify STARTBLOCK :value $currentstart
   :modify ENDBLOCK :value $currentend
   :modify MAXFILE :value $maxfilesize
    }
}
                 
proc click-volume v {
   global currentvolume
   set currentvolume "DISC"
   }

proc click-file f {
   global currentvolume currentfile
   set currentvolume "DISC"
   set currentfile  $f
   }

proc click-startblock s {
   global currentstart
   set currentstart  [string toupper $s]
   }

proc click-endblock e {
   global currentend
   set currentend  [string toupper $e]
   }

proc click-maxfile s {
   global maxfilesize soap_server preferences
   set maxfilesize  $s
   writereg outputfilesize $s

    set preferences([set soap_server]_maxfilesize) $s
    update_preferences

   repaint
   }

proc paint-maxfile-option {} {
   global maxfilesize
   :modify MAXFILE :value $maxfilesize
}


proc init-values {} {
   global debug maxfilesize
   return 0
   }


proc click-add {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   frame-busy
   set currentvolume DISC
   if { $currentfile == ""} {frame-idle; return}
   if { $currentstart == ""} {set currentstart 0}
   if { $currentend == ""} {set currentend 0}
   set runfileselected [llength $runfilelist]
   set l "$runfileselected $currentvolume $currentfile $currentstart $currentend"
   lappend runfilelist [format-runfile $l]
   paint-runfiles
   frame-idle
   }

proc click-insert {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   frame-busy
   if { $currentvolume == ""} {frame-idle; return}
   if { $currentfile == ""} {frame-idle; return}
   if { $currentstart == ""} {set currentstart 0}
   if { $currentend == ""} {set currentend 0}
   set l "$runfileselected $currentvolume $currentfile $currentstart $currentend"
    set runfilelist [linsert $runfilelist [expr $runfileselected +1] [format-runfile $l]]
   paint-runfiles
   frame-idle
   }

proc click-replace {} {
   global runfilelist runfileselected outhandler
   global currentvolume currentfile currentstart currentend
   frame-busy
   if {![llength $runfilelist]} then {
      click-add
      frame-idle
      return
      }
    if {$outhandler == 1} {set currentvolume DISC}
   set l "$runfileselected $currentvolume $currentfile $currentstart $currentend"
   set l [format-runfile $l]
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc start-handler args {
   global started currentvolume
   set currentvolume "DISC"
   if {![init-values]} then { 
      make-frame
      update idletasks
      click-refresh
      set started 1
      }
   }
