
set frame .sysoh
set framename .sysoh
set frametitle "TDR Output Handler Control"
set icontitle "OutputHandler"

set debug 0
set started 0
set sync 0

set usersetupdir [inform tdrFiles set usersetupdir]
set runfilename ""

set outhandler ""
set outhandlers "Null TapeServer Disc Tape"

set outputchannels "0 1 2 3"
foreach oc $outputchannels {
    set outputlabel($oc) "Output Channel $oc"
}

set handlerloaded ""
set handlercurrent -1
set handlersource ""
set handlerlist {}
set handlercount 0
set handler ""

set status "Not Loaded"

set tracelevel 0

set deviceselected 0

set runfile ""
set runfileselected -1
set runfilelist {}
set runfilecount 0

set currentvolume ""
set currentfile   ""
set currentstart  0
set currentend    0
set maxfilesize 2

set filelist {}

loadsource common.soap


#------------------------------------------------------------------------------

proc fopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	display-midas-error-report self "" $m
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
	if { ! [isComment? $t] } { lappend buf $t }
    }
    close $fp
    return $buf
}

proc write-file {f args} {
    set fp [fopen $f w]
    if {$fp==0} {return 1}
    foreach buf $args {
	foreach line $buf {
	    puts $fp $line
	}
    }
    close $fp
    set-footer "Saved to $f"
    return 0
}

proc last p {
   set n [llength $p]
   incr n -1
   return [lindex $p $n]
   }

proc front p {
   set n [llength $p]
   incr n -1
   return [lreplace $p $n $n]
   }

proc confirm {m action} {
   global debug
   set i [tk_dialog [self frame-name].quitdialog "[self frame-title] Confirmation" $m  "warning" 1 "Cancel" "Confirm"]
   if {$i} then $action
   if {$debug} then {puts stderr "confirm: $m choice $i"}
   }

proc click-handler-go {} {
   frame-busy
   set-footer "Starting handler ..."
   repaint
   frame-idle
   }

proc click-handler-stop {} {
   frame-busy
   set-footer "Stopping handler ..."
   repaint
   frame-idle
   }

proc click-help {} {inform tdrControl click-help}


proc click-resetline {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   if {![llength $runfilelist]} then {return}
   frame-busy
   set currentstart 0
   set currentend 0
   set l "$runfileselected $currentvolume $currentfile $currentstart $currentend"
   set l [format-runfile $l]
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc click-resetall {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   if {![llength $runfilelist]} then {return}
   frame-busy

    set currentstart 0
    set currentend 0

    set newlist ""
    set runfileselected 0
    foreach runfile $runfilelist {
	set runfileselected [first $runfile]
	set currentvolume [second $runfile]
	set currentfile [third $runfile]
	set l "$runfileselected $currentvolume $currentfile 0 0"
	lappend newlist [format-runfile $l]
	incr runfileselected
    }
   set runfilelist $newlist
   paint-runfiles
   frame-idle
   }

proc click-remove {} {
   global runfilelist runfileselected
   frame-busy
   if {![llength $runfilelist]} then {
      display-midas-error-report self "" "Runfile list is empty"
      frame-idle
      return
      }
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected]
   set runfileselected [expr $runfileselected - 1]
   if { $runfileselected < 1 } then {set runfileselected 0}
   paint-runfiles
   frame-idle
   }


proc click-save {} {
   update-program
   update-runfiles
   frame-idle
}

proc update-program {} {
   global runfilelist outchannel
   frame-busy
   set i 0
   set z 0
   set m ""
   writereg runfileindex 0
   foreach x $runfilelist {
       writereg outputrunfiledata[set outchannel] $x
       incr i
      }
    writereg outputrunfiledata[set outchannel] "$i END END 0 0"
    frame-idle
   }

proc click-runlist {c f} {
   global debug
   global runfile runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   if {$debug} then {puts stderr "click-list: $c $f"}
   clear-footer

   case $c in {
      :list-select {
         set runfile $f
         set runfileselected [first $runfile]
         set currentvolume [second $runfile]
         set currentfile [third $runfile]
         set currentstart [fourth $runfile]
         set currentend [fifth $runfile]
        }
     :list-deselect {
        set runfile ""
         set runfileselected 0
        }
     }          
   if {$runfileselected < 1} then { set runfileselected 0 }
   paint-runfiles
   }

proc update-runfiles {} {
    global runfilelist usersetupdir soap_server outchannel

    set usersetupdir [inform tdrFiles set usersetupdir]
    set r ""
    foreach line $runfilelist {lappend r $line}
    write-file $usersetupdir/outruninfo[set outchannel]_[set soap_server] $r
}

proc click-load-runfiles {} {
    global runfilelist usersetupdir soap_server outchannel

    set usersetupdir [inform tdrFiles set usersetupdir]
    set r [read-file $usersetupdir/outruninfo[set outchannel]_[set soap_server]]

    set runfilelist $r


    paint-runfiles
    update-program
}

proc load-handler-source {} {
   global handlerloaded debug

    set hs sysoh-[set handlerloaded]

   if {$debug} {puts stderr "Loading source for $hs ..."}
#   catch {destroy .sysoh}
   loadsource $hs
   set started 0
   start-handler
   set started 1
}

proc read-runinfo {} {
   global runfilename runfilelist runfileselected outchannel
   set runfilelist {}
   set i 0
   set z 0
   writereg runfileindex 0
   while {$z==0} {
      set info [readreg outputrunfiledata[set outchannel]]
       if {[first $info] == -1} {break}
       if {[first $info] == "?"} {break}
       if {[second $info] == 0} {continue}
       lappend runfilelist [format-runfile $info]
      }
   set runfileselected 0
   paint-runfiles
   }

proc format-runfile {r} {
   scan $r "%d %s %s %d %d" index s1 s2 i1 i2
   set r [format "%3d %6s %-30s %8d %8d" $index $s1 $s2 $i1 $i2]
   return $r
   }

proc click-refresh {} {
   frame-busy
   clear-footer
   read-runinfo
   repaint
   frame-idle
   }

proc finish {} {DeleteTask}

set regfile ""
set host ""
set port ""

proc sysoh.regfile_callback {r} {global regfile; set regfile $r}
proc sysoh.host_callback {r} {global host; set host $r}
proc sysoh.port_callback {r} {global port; set port $r}

proc start args {
   global started debug maxfilesize
   global outchannel outhandler outhandlers handlercurrent handlerloaded 
    global soap_server regfile host port

    set soap_server [first $args]

    set preferences([set soap_server]_maxfilesize) 2
    load_preferences
    set maxfilesize $preferences([set soap_server]_maxfilesize)

    inform $soap_server call-back sysoh sysoh.regfile_callback readregisterfile
    inform $soap_server call-back sysoh sysoh.host_callback readhost
    inform $soap_server call-back sysoh sysoh.port_callback readport

    define_registers $regfile
    server-connect $soap_server

    set outhandler [second $args] 
    set outchannel [third $args]
    set handlerloaded [lindex $outhandlers $outhandler]


   if {$outhandler != $handlercurrent} {
      load-handler-source
      set handlercurrent $outhandler
      set started 1
   } {
      if {$started} then {
	  click-refresh
	  bring-to-front [self frame-name]
      }
   }
}
