
set debug 0

set framename .tdrcontrol

set frame .tdrsetup
set frametitle "TDR Event Builder Control"
set icontitle "TDR EB"

set FilterEnabled 1
set EBEnabled 1

loadsource tdrDefaults
if { ![info exists env(TDRBASE)] } {set env(TDRBASE) $defaultTDRBASE}

set basedir [set env(TDRBASE)]
set usersetupdir ""

loadsource common.soap
set soap_server tdrEB

switch $env(os) {
    SunOS    {
	switch -glob $env(machine) {
	    i86pc   {set env(DIRPOSTFIX) ""}
	    default {set env(DIRPOSTFIX) "SunOS"}
	}
    }
    Linux  {
	set kernel [exec uname -r]
	switch -glob $kernel {
	    2.6.*    -
	    default {set env(DIRPOSTFIX) "Linux"}
	}
	switch -glob $env(MACHTYPE) {
	    i*86   {set env(DIRPOSTFIX) "Linux"}
	    *64    {set env(DIRPOSTFIX) "Linux64"}
	    default {set env(DIRPOSTFIX) "Linux"}
	    }
    }
    Linux64  {
	set env(DIRPOSTFIX) "Linux64"
    }
    default {
	set env(DIRPOSTFIX) ""
    }
}
if {[set env(DIRPOSTFIX)] == ""} {
    set-footer "Architecture not supported"
    return
}

set env(PATH) $env(TDRBASE)/bin_[set env(DIRPOSTFIX)]:$env(PATH)
set env(LD_LIBRARY_PATH) $env(TDRBASE)/lib_[set env(DIRPOSTFIX)]:$env(LD_LIBRARY_PATH)


set bindir $basedir/bin_$env(DIRPOSTFIX)
set libdir $basedir/lib_$env(DIRPOSTFIX)

if {[string first $libdir [set env(LD_LIBRARY_PATH)]] == -1} {
    set env(LD_LIBRARY_PATH) $libdir:[set env(LD_LIBRARY_PATH)]
}
if {[string first $bindir [set env(PATH)]] == -1} {
    set env(PATH) $bindir:[set env(PATH)]
}

set bitmaps $basedir/tcl/bitmaps
set configdir $basedir/config

set fileloaded 0
set execloaded 0
set fexecloaded 0

set preferences(filter_enable) 0
set preferences(eb_enable) 1
load_preferences
set filter_enable $preferences(filter_enable)
set eb_enable $preferences(eb_enable)

set state ""
set states(0) STOP
set states(1) GO
set statestring(0) STOPPED
set statestring(1) GOING

# ============================================================================

proc log-put {str} {
    :modify LOG :strings $str\n
}

proc log-empty {} {
    if { [winfo exists [self frame-name]] } {
	bring-to-front [self frame-name]
    } {
	start
    }
    
    :modify LOG :empty T
}

# ============================================================================

proc click-display-spectra {} {
   global debug
   if {$debug} {puts stderr "click-display-spectra"}
   frame-busy
   clear-footer
   inform sb sortstart 1
   frame-idle
   }

proc click-refresh {} {
    global fileloaded filter_enable eb_enable
    global execloaded fexecloaded FilterEnabled EBEnabled

    if {$eb_enable} {set se [readreg state]} {set se "Not Enabled"}

    if {$execloaded} {
	    loadprog_disable	
    } {
	if {$se == "?"} {
	    set se "Not Loaded"
	    if {$eb_enable} { install_disable }
	} {
	    if {$EBEnabled} {
		install_enable
		loadprog_enable
	    }
	}
    }

    if {$filter_enable} {set sf [inform tdrFilter read-state]} {set sf "Not Enabled"}
    if {$fexecloaded} {
	floadprog_disable	
    } {
	if {$sf == "?"} {
	    set sf "Not Loaded"
	    if {$filter_enable} { install_disable }
	} {
	    if {$FilterEnabled} {
		install_enable
		floadprog_enable
	    }
	}
    }
	
    set z [catch {set fileloaded [inform tdrFiles set fileloaded]} m]
    if {$z} {set fileloaded 0}
    if {[string first can $fileloaded] != -1} {set fileloaded 0}
    if {$fileloaded} {install_enable}

    if {$EBEnabled} { :modify ESTATE :value $se }
    if {$FilterEnabled} { :modify FSTATE :value $sf }

    if {$eb_enable} {
	if {$filter_enable} {set st $sf} {set st $se}
    } {
	if {$filter_enable} {set st $sf } {set st "Not Loaded"}
    }

    if {$se == "GOING"} {set v 1} {set v 0}
    :modify STATE :value $st
    :modify GOSTOP :value $v

}

proc click-flenable {v} {
    global filter_enable preferences
    set filter_enable $v
    set preferences(filter_enable) $v
    update_preferences

    if {$filter_enable} {set deaf F} {set deaf T}
    :modify FILTER :inactive $deaf
    click-refresh
}

proc click-ebenable {v} {
    global eb_enable preferences
    set eb_enable $v
    set preferences(eb_enable) $v
    update_preferences
    click-refresh
}

proc click-filter {} {
    global soap_server
    inform tdrFilter start
}

proc click-ebcontrol {} {
    global soap_server
    inform $soap_server start
}

proc click-loadfile {} {
    global soap_server fileloaded execloaded
    global filter_enable eb_enable
    inform tdrFiles start $soap_server

    set z [catch {set fileloaded [inform tdrFiles set fileloaded]} m]
    if {$z} {set fileloaded 0}
    if {[string first can $fileloaded] != -1} {set fileloaded 0}

    modifysetup_enable
    if {$eb_enable == 0} {
	if {$filter_enable != 0} {install_enable}
   } { 
	if {$execloaded} {install_enable}
    }
}

proc click-setup {} {
    global soap_server
    inform tdrTree start $soap_server
}

proc click-help {} {
    global basedir
    clear-footer
    set f [list "" $basedir/manual/tdr.html]
    inform help html_show $f
}

proc click-statistics {server} {
    clear-footer
    set regfile [inform $server set register_file]
puts "statistics start $server $regfile"
    inform [set server]stats start $server $regfile
}

proc click-savespectra {} {
   global debug filter_enable eb_enable
   if {$debug} then {puts stderr "click-save-spectra"}
   frame-busy
   clear-footer
    if {$filter_enable} {
	inform tdrFilter click-savespectra
    }
    if {$eb_enable} {
	inform tdrEB click-savespectra
    }
   frame-idle
   }

proc gostop-filter {v} {
    global states statestring filter_enable

    if {$filter_enable} {
	set sf [inform tdrFilter read-state]
	if {$sf == "?"} {set-footer "Filter Program not loaded"; return}

	set z [inform tdrFilter write-state $v]
	if {$z} {set-footer "Filter program failed"} {clear-footer}
	
	set i 20
	while {$i} {
	    after 500
	    set sf [inform tdrFilter read-state]
	    if {$sf == "?"} {break}
	    if {$sf == [set statestring($v)]} {break}
	    incr i -1
	}
	if {$sf != [set statestring($v)]} {
	    set-footer "Program in wrong state for [set states($v)]"
	}
    } {set sf "Not Enabled"}

#    puts "gostop-filter $v inform tdrFilter refresh"
    inform tdrFilter refresh
    return $sf
}

proc gostop-eb {v} {
    global states statestring eb_enable

    if {$eb_enable} {
	set se [readreg state]
	if {$se == "?"} {set-footer "Program not loaded"; return}

	set z [catch {writereg state [set states($v)]} m]
	if {$z} {set-footer $m} {clear-footer}
	
	set i 10
	while {$i} {
	    after 500
	    set se [readreg state]
	    if {$se == "?"} {break}
	    if {$se == [set statestring($v)]} {break}
	    incr i -1
	}
	if {$se != [set statestring($v)]} {
	    set-footer "Program in wrong state for [set states($v)]"
	}
    } {set se "Not Enabled"}

    inform tdrEB refresh
    return $se
}

proc click-gostop {v} {
    global statestring FilterEnabled EBEnabled

    if {$v} {
	set se [gostop-eb $v]
	set sf [gostop-filter $v]
    } { 
	set sf [gostop-filter $v]
	set se [gostop-eb $v]
    }

    if {$FilterEnabled} {:modify FSTATE :value $sf}
    if {$EBEnabled} {:modify ESTATE :value $se}
    :modify STATE :value [set statestring($v)]
    :modify GOSTOP :value $v
}

proc loadprog_enable {} { [:path eloadprog] configure -state normal }
proc loadprog_disable {} { [:path eloadprog] configure -state disabled }
proc install_enable {} { [:path install] configure -state normal }
proc install_disable {} { [:path install] configure -state disabled }
proc loadsetup_enable {} { [:path loadfile] configure -state normal }
proc loadsetup_disable {} { [:path loadfile] configure -state disabled }
proc modifysetup_enable {} { [:path SETUP] configure -state normal }
proc modifysetup_disable {} { [:path SETUP] configure -state disabled }

proc click-eloadprog {} {
    global bindir execloaded soap_server
    loadprog_disable
    inform $soap_server loadexecprog $bindir TDRmain
    set execloaded [inform $soap_server set execloaded]
     if {$execloaded} {
	loadsetup_enable
	loadprog_disable
	install_enable
     } {
	 loadprog_enable
     }
    click-refresh
}

proc click-ekillprog {} {
    global bindir execloaded soap_server
    inform $soap_server killexecprog $bindir TDRmain
    set execloaded 0
    install_disable
#    loadsetup_disable
    loadprog_enable
    click-refresh
}

proc floadprog_enable {} { [:path FLOADPROG] configure -state normal }
proc floadprog_disable {} { [:path FLOADPROG] configure -state disabled }

proc click-floadprog {} {
    global bindir fexecloaded
    floadprog_disable
    inform tdrFilter loadexecprog $bindir TDRfilter
    set fexecloaded [inform tdrFilter set execloaded]
    if {$fexecloaded} {
	loadsetup_enable
	floadprog_disable
	install_enable
     } {
	 floadprog_enable
     }
    click-refresh
}

proc click-fkillprog {} {
    global bindir fexecloaded
    inform tdrFilter killexecprog $bindir TDRfilter
    set fexecloaded 0
    inform tdrFilter set execpid 0
    install_disable
#    loadsetup_disable
    floadprog_enable
    click-refresh
}

#------------------------------------------------------------------------------

set apparatus_name      ""
set resource_name       ""
set resource_cap        ""
set resource_port       ""
set resource_host       ""
set resource_role       ""
set resource_type       ""
set resource_app        ""

proc get-resources {subsystem} {
    global resources apparatus_name resource_cap
    global resource_name resource_host resource_role
    global resource_type resource_app resource_port

    set resources [ :experiment resources ]

    set found 0
    foreach res $resources {
       foreach r $res {
          switch [ first $r ] {
            name          { set resource_name [ second $r ] }
            host          { set resource_host [ second $r ] }
            role          { set resource_role [ second $r ] }
            type          { set resource_type [ second $r ] }
            apparatus     { set resource_app  [ second $r ] }
            port          { set resource_port [ second $r ] }
            capability    { set resource_cap  [ second $r ] }
          }
       }

	if { $resource_type == "spectrum" && $resource_port == 10235 && $resource_role == $subsystem } {
	    set found 1; 
	    inform ap start
	    inform ap wm iconify .ap
#	    inform ap set apparatus $resource_app
#	    set apparatus_name $resource_app
	    set apparatus_name tdrshm
	    break;
	}
    }
    puts "get-resources: ret $found [set apparatus_name] [set resource_role] [set resource_cap]"
    return $found
}

proc update_shmsas_capability {subsystem} {
    global apparatus_name
    set found [get-resources $subsystem]
    if {$found} {
	set z [catch {eg "(apparatus :name '$apparatus_name' !free)"} m]
	set z [catch {eg "(apparatus :name '$apparatus_name' !claim)"} m]
    } {
	set-footer " Shmsas resource $subsystem not found"
    }
}

#------------------------------------------------------------------------------

proc click-install {} {
    global filter_enable eb_enable

    log-empty

    set usersetupdir [inform tdrFiles set usersetupdir]
    inform tdrFiles click-save currentsetup

    if {![file isfile $usersetupdir/currentsetup.cdef]} {
	set-footer "Cluster setup file not defined"
	return
    }

    if {![file isfile $usersetupdir/currentsetup.ndef]} {
	set-footer "EventSearch setup file not defined"
	return
    }


    set-footer "Setup installation starting ..."

    if {$filter_enable} {
#	update_shmsas_capability filter

	set z [inform tdrFilter spectrum-directory-ok]
	if {!$z} {
	    set-footer "Spectrum directory error"
	    return
	}
	inform tdrFilter click-install
    }
    if {$eb_enable} {
#	update_shmsas_capability eb

	set z [inform tdrEB spectrum-directory-ok]
	if {!$z} {
	    set-footer "Spectrum directory error"
	    return
	}
	inform tdrEB click-install
    }

    click-refresh
}


set font -*-Helvetica-Bold-R-Normal--13-120-*-*-*-*-iso8859-1

proc make-frame {} {
    global state font frametitle
    global loadboth listtypechoice dirlisttype
    global FilterEnabled EBEnabled filter_enable eb_enable

    set xpos 18c
    set xpos1 15c
   set xpos3 25c

    set twidth 39
   set twidth2 19
    set lwidth 200

    :frame [self frame-name] :label $frametitle :show-footer T \
	:icon-name "[self icon-title]" :resize F
    [:path [self frame-name]] configure -background LightSlateGrey
   
   :container ACQ  :layout T :anchor N :relief R :padx 3 :pady 3

    :panel P1 :layout T :pady 2 :padx 2
    :layout H

    :choice GOSTOP :label "Acquisition" :strings {Stop Go} :valuex $xpos1 :w 8
    [:path GOSTOP].button0 configure -selectcolor red -height 2
    [:path GOSTOP].button1 configure -selectcolor green -height 2

    :panel P2 :layout T :pady 2 :padx 2
    :layout H
    :text STATE :label "System Status:" :value $state \
	:x 20 :w $twidth2

    [:path STATE].entry configure -background cornsilk

    :next-row T
    :button SAVESPECTRA :label "Save All Shm Spectra to disc" :x 20

#    :choice-stack BLOCKSIZE :label "BlockSize:" :x 450  \
#	:menu $blocksizestrings 
#    [:path BLOCKSIZE].message configure -text $blocksize

    pack [:path P1] -side left
    pack [:path P2] -side right

   :container MODIFYSETUP  :layout T :anchor N :relief R :padx 3 :pady 3
    :panel P3 :layout T :pady 2 :padx 2 
    :layout H; :col-gap 50

    :button LOADFILE :x 40 :w 12 :label "Load/Save Setup"

    :button SETUP :w 12 :label "Modify Setup" :x 160
 
    :button INSTALL :w 12 :label "Install" :x 280

    if {$FilterEnabled} {

    :container FCONTROL  :layout T :anchor N :relief R :padx 3 :pady 3
    :panel P4 :layout T :pady 3 :padx 2 
    :layout H; :row-gap 5

    :checkbox FLENABLE :label "" :value $filter_enable
    if {$filter_enable} {set deaf F} {set deaf T}
    :button FILTER :x 40 :w 12 :label "Filter Control" :inactive $deaf
    :button FLOADPROG :w 12 :label "Load Prog" :x 160
    :button FKILLPROG :w 12 :label "Exit Prog" :x 280

    :text FSTATE :label "" :x 380 :w $twidth2
    [:path FSTATE].entry configure -background cornsilk
    
    }
    if {$EBEnabled} {

    :container ECONTROL  :layout T :anchor N :relief R :padx 3 :pady 3
    :panel P5 :layout T :pady 2 :padx 2 
    :layout H; :row-gap 5

    :checkbox EBENABLE :label "" :value $eb_enable
    :button EBCONTROL :x 40 :w 12 :label "EB Control"
    :button ELOADPROG :w 12 :label "Load Prog" :x 160
    :button EKILLPROG :w 12 :label "Exit Prog" :x 280
 
    :text ESTATE :label "" :x 380 :w $twidth2
    [:path ESTATE].entry configure -background cornsilk

     }

    :container IN :layout T :anchor N :padx 3 :pady 3

    :panel P6 :layout T :anchor CE :fill NONE :pady 5

    :galley LOG :rows 6 :cols 60


    :panel BUTTONS  :layout T :padx 10 :pady 2 
      :layout H; :row-gap 20; :col-gap 10

    if {$FilterEnabled} {
	:button FSTATISTICS :label "Filter Statistics" :w 12 :x 10
    }
    if {$EBEnabled} {
	:button ESTATISTICS :label "EB Statistics" :w 12
    }
    :button DISPLAYSPECTRA :label "Display spectra" :w 12
    :button REFRESH :label "Refresh" :w 8
    :button MYHELP    :label "Help"

    pack [:path MYHELP] -side right
    pack [:path REFRESH] -side right
    pack [:path DISPLAYSPECTRA] -side right

    update
    :show T
}

proc click arg {
   global debug chan
   if {$debug} then {puts stderr "myclick with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit $f; return}
   switch -glob $w {

      MYHELP         {click-help}
      REFRESH        {click-refresh}

      ELOADPROG       {click-eloadprog}
      EKILLPROG       {click-ekillprog}
      FLOADPROG       {click-floadprog}
      FKILLPROG       {click-fkillprog}
      INSTALL        {click-install}
      LOADFILE       {click-loadfile}
      SETUP          {click-setup}
      FILTER         {click-filter}
      EBCONTROL      {click-ebcontrol}
      FLENABLE       {click-flenable $v}
      EBENABLE       {click-ebenable $v}

      GOSTOP         {click-gostop $v}
      SAVESPECTRA    {click-savespectra}
      FSTATISTICS    {click-statistics tdrFilter}
      ESTATISTICS    {click-statistics tdrEB}
      DISPLAYSPECTRA {click-display-spectra}
      default        {puts stdout $arg}
   }
}


proc start {} {
    global soap_server started fileloaded
    global filter_enable eb_enable

    if {$started} {
	bring-to-front [self frame-name]
	return
    } {
	inform tdrFilter start
#	if {$filter_enable == 0} {inform tdrFilter wm iconify .tdrfilter}
	inform tdrFilter wm iconify .tdrfilter
	inform tdrEB start
#	if {$eb_enable == 0} {inform tdrEB wm iconify .tdreb}
	inform tdrEB wm iconify .tdreb
	set regfile [inform tdrEB set register_file]
	define_registers $regfile
	server-connect tdrEB
        make-frame

	inform tdrFiles start $soap_server
	inform tdrFiles wm iconify .tdrfiles

        set z [catch {set fileloaded [inform tdrFiles set fileloaded]} m]
        if {$z} {set fileloaded 0}
        if {[string first can $fileloaded] != -1} {set fileloaded 0}
        if {$fileloaded} {
            set z [catch {set setupfile [inform tdrFiles set setupfile]} m]
	    log-put "Setup file $setupfile loaded"
	    install_enable
	}
    }


    click-refresh

    set started 1
}