set framename .tdrfiles

set frame .tdrfiles
set frametitle "TDR Event Builder"
set icontitle "TDR EB"

loadsource tdrDefaults
if { ![info exists env(TDRBASE)] } {set env(TDRBASE) $defaultTDRBASE}

set basedir [set env(TDRBASE)]

set loadboth 2
set dirlisttype 1
set listtypechoice {"Sys Setups" "User Directory"}
set fileloaded 0

set syssetupdir $basedir/setups

set preferences(usersetupbase) $DefaultSetupBase
set preferences(usersetupdir) $DefaultSetupBase/$DefaultSetupDir
set preferences(savedir) $preferences(usersetupdir)
set preferences(setupfile) "currentsetup"
load_preferences
set usersetupbase $preferences(usersetupbase)
set usersetupdir $preferences(usersetupdir) 
set savedir $preferences(savedir)
set setupfile $preferences(setupfile)

loadsource common.soap

proc click-list-type {n} {
   global debug listtypechoice dirlisttype syssetupdir usersetupdir savedir
   if {$debug} then {puts stderr "click-list-type: $n"}
   set dirlisttype $n

    switch $dirlisttype {
	0 {set savedir $syssetupdir}
	1 {set savedir $usersetupdir}
    }

   read-directory
   paint-directory
}

proc dirnames l {
   set ll {}
   foreach f $l {lappend ll [first $f]}
   return $ll
   }

proc create-dir {newdir} {
   global errorCode
	puts "Creating $newdir..."
   set z [catch {exec mkdir $newdir} m]
   if {$z!=0 && $errorCode!="NONE"} then {
      display-midas-error-report self "" \
              "Attempting to create directory $newdir:\n" $m
      error $m
      }
   }

proc read-directory {} {
    global debug
    global savedir setupfiles
    # improve error for case where directory does not exist
    if {$debug>1} {puts stderr "read-directory"}
    set z [catch {eg "(directory :path '$savedir' ?files)"} m]
    if {$z!=0} then {
	display-midas-error-report self $m \
	    "Attempting to read directory" $savedir
	error $m
    }
    set files [lsort [strip-dots [dirnames $m]]]
    set sfiles ""
    foreach s $files {
	set n [string last . $s]
	if {$n == -1} {continue}
	set s2 [string range $s $n end]
	incr n -1
	set s1 [string range $s 0 $n]
	if {$s2 == ".cdef"} {lappend sfiles $s1}
	if {$s2 == ".ndef"} {lappend sfiles $s1}
    }
    set setupfiles [lsort -unique $sfiles]
}

proc get-setupfiles {} {
    read-directory
}

proc MakeSetupFrame {} {
    global syssetupdir usersetupdir savedir loadboth listtypechoice
    global setupfile setupfiles dirlisttype

    set xpos 18c
    set rows 12
    set twidth 39
    set lwidth 200

    :frame [self frame-name] :label "TDR Event Builder Setup" :show-footer T \
	:icon-name "[self icon-title]" :resize F
    
    :panel LISTING  :layout R :padx 10 :pady 10
      :layout H; :row-gap 10; :col-gap 5
      :choice LIST-TYPE   :strings $listtypechoice :value $dirlisttype \
             :bold T :x 40
      :next-row T
      :list FILELIST  :rows $rows :list-width $lwidth :x 0 :exclusive T \
	:strings $setupfiles :double-click T
      [:path FILELIST].list configure -background white
      :next-row T
      :button DELETEFILE  :label "Delete"
      :button REFRESH :label "Refresh"

    :panel FILEING  :layout T :padx 10 :pady 10 
      :layout H; :row-gap 19; :col-gap 10

      :next-row T
      :next-row T
      :text SETUPFILEDIR :label "Setupfiles directory:" :value $syssetupdir \
	:valuex $xpos :w $twidth
      [:path SETUPFILEDIR].entry configure -background white
      :next-row T
      :text SETUPFILE :label "Setupfile:" :value $setupfile :valuex $xpos :w $twidth
      [:path SETUPFILE].entry configure -background white
       :next-row T
      :text SETUPDIR :label "Setup directory:" :value $usersetupdir \
	:valuex $xpos :w $twidth
      [:path SETUPDIR].entry configure -background white
      :next-row T
        :choice LOADBOTH :label "Load setup files"  :exclusive T :strings [list Cluster  EvSearch Both] :value 2 :command click-loadboth
      :next-row T

    :button MTSORTMAPS :label "Generate MTsort Maps"  :command click-mtsortmaps

      :next-row T
      :button LOAD :label "Load File"
      :button SAVE :label "Save current Setup"
      :button ZAP-SETUP :label "Reset"
      :button MYHELP    :label "Help"

    bind  [:path FILELIST] <Double-Button-1> double-click-filelist

    click-list-type $dirlisttype
    :show T
}

proc double-click {arg} {
   global debug WIDGET COMMAND FRAME VALUE ITEM

   set f [lindex $arg 0]
   set FRAME $f
   set c [lindex $arg 1]
   set COMMAND $c
   set w [lindex $arg 2]
   set WIDGET $w
   set v [lindex $arg 3]
   set VALUE $v
   set i [lindex $arg 4]
   set ITEM $i
   set p "double-click-[lowercase $w]"

   if {[info procs $p] == "$p"} then {
      set argnum [llength [info args $p]]
      if {$argnum == 0} then {$p}
      if {$argnum == 1} then {$p "$v"}
      if {$argnum == 2} then {$p "$v" "$i"}
      if {$argnum > 2} then {puts "double-click: $p expects too many args ($argnum)"}
   } else {
      set p "double-click-default"
      if {[info procs $p] == "$p"} then {
         set argnum [llength [info args $p]]
         if {$argnum == 0} then {$p}
         if {$argnum == 1} then {$p $w}
         if {$argnum == 2} then {$p $w "$v"}
         if {$argnum == 3} then {$p $w "$v" "$i"}
         if {$argnum > 3} then {insert-debug "double-click: $p expects too many args ($argnum)"}
      } else {
         puts "double-click with argument \{$arg\}"
      }
   }
}

proc myclick arg {
   global debug chan
    clear-footer
   if {$debug} then {puts stderr "myclick with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit $f; return}
   switch -glob $w {

      LIST-TYPE      {click-list-type $v}
      FILELIST       {click-filelist $c $v}

      UP             {click-up}
      MYHELP         {click-myhelp}
      DELETEFILE     {click-deletefile}
      REFRESH        {click-refresh}

      SETUPFILEDIR   {click-setupfiledir $v}
      SETUPDIR       {click-setupdir $v}
      SETUPFILE      {click-setupfile $v}

      ZAP-SETUP      {click-zap-setup}
      LOAD           {click-load}
      SAVE           {click-save ""; click-refresh}

      default        {puts stdout $arg}
   }
}

proc double-click-filelist {v i} {click-filelist $v $i;click-load}

proc update-setupfile {s} {
    global preferences
    set preferences(setupfile) $s
    update_preferences
}

proc click-filelist {v i} {
    global debug
    global setupdir savedir setupfile
    
    if {$debug} then {puts stderr "click-filelist: $v $i"}
    clear-footer
    
    if {[file isfile $savedir/$v.cdef] || [file isfile $savedir/$v.ndef]} {
	set setupfile $v
	:modify SETUPFILE :value $setupfile
    } else {
	display-midas-error-report self "" "Cannot select setupfile:\n" \
	    "$v is not a file or directory"
	set setupfile ""
    }
    update-setupfile $setupfile
}

proc click-setupfile {f} {
    global setupfile
    clear-footer
    set setupfile $f
    update-setupfile $setupfile
   }

proc click-setupfiledir {f} {
   global syssetupdir dirlisttype savedir
   clear-footer
   set syssetupdir $f
    if {!$dirlisttype} {set savedir $f}
   click-refresh
   }

proc click-setupdir {f} {
   global errorCode usersetupdir dirlisttype savedir preferences
   clear-footer

    if {![file exists $f]} {
	set z [catch {exec mkdir $f} m]
	if {$z!=0 && $errorCode!="NONE"} {
	    display-midas-error-report self "" \
		"Attempting to create directory $f:\n" $m
	    error $m
	} {
	    set usersetupdir $f
	    writereg setupdir $f
	}
    } {
	set usersetupdir $f
	writereg setupdir $f
    }
    
    if {$dirlisttype == 1} {set savedir $f}

    set preferences(usersetupdir) $usersetupdir
    set preferences(savedir) $savedir
    update_preferences

   click-refresh
}

proc confirm {m action} {
   global debug
   set i [tk_dialog [self frame-name].quitdialog "[self frame-title] Confirmation" $m  "warning" 1 "Cancel" "Confirm"]
   update idletasks
   if {$i} then $action
   if {$debug} then {puts stderr "confirm: $m choice $i"}
   }

proc click-deletefile {} {
    global debug
    global setupfile savedir
    
    if {[file isfile $savedir/$setupfile.cdef]} {
	confirm "  Delete setupfile   $setupfile.cdef ?  " "complete-deletefile $setupfile.cdef"
    }
    
    if {[file isfile $savedir/$setupfile.ndef]} {
	confirm "  Delete setupfile   $setupfile.ndef ?  " "complete-deletefile $setupfile.ndef"
    }
    
    frame-idle
}

proc complete-deletefile {f} {
   global debug
   global errorCode
   global savedir

    if {$debug} {puts stderr "complete-delete: setupfile=$f"}
    
    set target $savedir/$f
    set z [catch {exec rm $target} m]
    if {$z!=0 && $errorCode!="NONE"} then {
	display-midas-error-report self "" "Delete failed for $target:\n" $m
	frame-idle
    } else {
	clear-footer
	read-directory
	# reset string to next one in current list
      paint-directory
	frame-idle
	set-footer "$target deleted."
    }
}

proc click-myhelp {} {
    global basedir
    send help html_show {"" $basedir/tcl/manual/sortsetup.html}
}

proc click-load {} {
    global setupfile savedir loadboth fileloaded

    set z [inform tdrTree click-loadsetupfiles $savedir/$setupfile $loadboth]
    if {$z==1} {set fileloaded 1} {set fileloaded 0}
    if {$fileloaded} {
	inform tdrTree redraw_nodes
	set-footer "Setup $setupfile loaded"
    }
}

proc click-save {saveto} {
    global setupfile savedir dirlisttype

    if {!$dirlisttype} {
	set-footer "Can only save to user directory ..."
	return
    }

    if {$saveto == ""} {
	set s $setupfile
	if {[string length $s] <1} {
	    set-footer "No setup file chosen"
	    return
	}
	set n [string last . $s]
	if {$n == -1} {
	    set n end
	} {
	    incr n -1
	    if {$n < 1} {
		set-footer "No setup file chosen"
		return
	    }
	}
	set f [string range $s 0 $n]
    } {
	set f $saveto
    }

    inform tdrTree click-savesetupfiles $savedir/$f

    inform tdrTree generate_mapfile $savedir

    set-footer "Setup saved to $savedir/$f.*"
}

proc click-loadboth {w v} {
    global loadboth
    set loadboth $v
}
    
proc click-mtsortmaps {} {
    global usersetupdir
    set-footer "Please wait ..."
    inform tdrTree generate_mapfile $usersetupdir
    set-footer Done
}

proc paint-directory {} {
   global debug setupfiles dirlisttype
   if {$debug} {puts stderr "paint-directory: "}
    catch {:modify FILELIST :empty T :strings $setupfiles :deselect -1}
    catch {:modify LIST-TYPE :value $dirlisttype}
   }

proc click-refresh {} {
   global debug
   if {$debug} then {puts stderr "click-refresh"}
   read-directory
   paint-directory
   }

proc click-quit f {
   global env debug
   if {$debug} then {puts stderr "click-quit with argument \{$f\}" }
   destroy-frame $f
}

proc draw-setup-frame {} {
    read-directory
    MakeSetupFrame 
}

proc start {server} {
    global started syssetupdir usersetupbase usersetupdir

    if {$started} {
       bring-to-front [self frame-name]
	return
    } {
	set regfile [inform $server set register_file]
	define_registers $regfile
	server-connect $server
    }

   if {![file exists $usersetupbase]} {
      create-dir $usersetupbase
   }
   if {![file exists $usersetupdir]} {
       create-dir $usersetupdir
       exec cp [set syssetupdir]/currentsetup.cdef $usersetupdir
       exec cp [set syssetupdir]/currentsetup.ndef $usersetupdir
   }

    draw-setup-frame

    if {! $started} {click-load}

    set started 1
}
