#!tree_wish -f
# -*-Tcl-*-
#
# dirtree11 - same as dirtree10, but adds bindings for mouse button <2>
#             to open a listbox displaying the files in the selected directory

set err [catch {load {} Tree} m]
if {$err} {
if {[catch {package require Tree} msg]} {
    puts "error loading Tree package $msg"
    return 1
}
}

set debug 0

set framename .tdr

set frame .tdr
set frametitle "TDR Event Builder"
set icontitle "TDR EB"

# Set same as in code ...
set MAXNODES   64
set MAXACTIONS 16

switch $env(os) {
    SunOS    {
	switch -glob $env(machine) {
	    i86pc   {set env(DIRPOSTFIX) ""}
	    default {set env(DIRPOSTFIX) "SunOS"}
	}
    }
    Linux  {
	set kernel [exec uname -r]
	switch -glob $kernel {
	    2.6.*    -
	    default {set env(DIRPOSTFIX) "Linux"}
	}
	switch -glob $env(MACHTYPE) {
	    i*86   {set env(DIRPOSTFIX) "Linux"}
	    *64    {set env(DIRPOSTFIX) "Linux64"}
	    default {set env(DIRPOSTFIX) "Linux"}
	}
    }
    Linux64  {
	set env(DIRPOSTFIX) "Linux64"
    }
    default {
	set env(DIRPOSTFIX) ""
    }
}
if {[set env(DIRPOSTFIX)] == ""} {
    set-footer "Architecture not supported"
    return
}

loadsource tdrDefaults
if { ![info exists env(TDRBASE)] } {set env(TDRBASE) $defaultTDRBASE}

set env(PATH) $env(TDRBASE)/bin_[set env(DIRPOSTFIX)]:$env(PATH)
set env(LD_LIBRARY_PATH) $env(TDRBASE)/lib_[set env(DIRPOSTFIX)]:$env(LD_LIBRARY_PATH)


set basedir [set env(TDRBASE)]

set templatepath $basedir/config/templates
set bitmaps $basedir/tcl/bitmaps


set clustersetupname default
set nodesetupname default

set menuoptions "-tearoff false -bg WhiteSmoke -fg blue -activebackground blue -activeforeground WhiteSmoke"
set menubuttonoptions "-relief raised -background MintCream -activebackground WhiteSmoke"

loadsource clustersetup
loadsource clusterwgen
loadsource setupparams
loadsource variables
loadsource dirtree10
loadsource mtsort_maps

set nodebase(name) "AddSearch"
set nodebase(up) {}
set nodebase(actions) {}

set gottext ""
set gotsel 0

set nodepathkeep ""
set actionpathkeep ""
set keepmode "Cut"

set cresult ""
set eresult ""

set histranges "256 1024 4096 8192 16384"
set histscales "0 1 2 3 4 5 6 7"
set tacdelays "8000 12000 4000 1000"
set outputchannels "1 2"
set detector_fold "1 2 3"

set timeranges {tk us ms sc}
set timescales "1 100 100000 100000000"

#set outputstyles {"Clusters_Filtered" "Clusters_Filtered_PlusAll"}
set outputstyles {"TDR_Events" "Eurogam_Events"}

# Register functions ...

set soap_server ""
loadsource common.soap

proc init_soap {server} {
    set regfile [inform $server set register_file]
    define_registers $regfile
    server-connect $server
}

# Command functions ...

proc get-node-from-name {name} {
    global MAXNODES
    set nn 1
    while {$nn < $MAXNODES} {
	set node node$nn
        global $node
	set names [array names $node]
	if {$names == ""} {incr nn;continue}
	set nname [set [set node](name)]
	if {$name == $nname} { return $node}
	incr nn
    }
    return ""
}

proc zap-node {node} {
    global $node
    set names [array names $node]
    foreach name $names {
	unset [set node]($name)
    }
}

proc actionmenu-Cut {} {
    global canvas tree nodepathkeep actionpathkeep
    set path [GetPath $canvas]
    if {$path == ""} {return}

    set nodepathkeep ""
    set actionpathkeep $path
    set act [file tail $actionpathkeep]
    set node [file tail [file dirname $actionpathkeep]]
    #puts "actionmenu-Cut $act $node"
    global $node

    set acts ""
    foreach a [set [set node](actions)] {
	if {$a != $act} {append acts "$a "}
    }
    set [set node](actions) $acts
    $tree rmlink $actionpathkeep
    zap-node [set node][set act]
    redraw_nodes
}

proc actionmenu-Copy {} {
    global canvas tree nodepathkeep actionpathkeep
    set path [GetPath $canvas]
    if {$path == ""} {return}

    set nodepathkeep ""
    set actionpathkeep $path
}

proc actionmenu-MoveUp {} {
    global canvas tree nodepathkeep actionpathkeep
    set path [GetPath $canvas]
    if {$path == ""} {return}

    set nodepathkeep ""
    set actionpathkeep $path
    set act [file tail $actionpathkeep]
    set node [file tail [file dirname $actionpathkeep]]
    global $node

    set acts [set [set node](actions)]
    set lacts [split $acts]
    if {[first $lacts] == ""} {set lacts [lreplace $lacts 0 0]}
    set p [lsearch $lacts $act]
    if {$p > 0} {
	set lacts [lreplace $lacts $p $p]
	incr p -1
	set lacts [linsert $lacts $p $act]
    }
    set [set node](actions) [concat $lacts]
    redraw_nodes
}

proc actionmenu-MoveDown {} {
    global canvas tree nodepathkeep actionpathkeep
    set path [GetPath $canvas]
    if {$path == ""} {return}

    set nodepathkeep ""
    set actionpathkeep $path
    set act [file tail $actionpathkeep]
    set node [file tail [file dirname $actionpathkeep]]
    global $node

    set acts [set [set node](actions)]
    set lacts [split $acts]
    if {[first $lacts] == ""} {set lacts [lreplace $lacts 0 0]}
    set p [lsearch $lacts $act]
    if {$p < [llength $acts]} {
	set lacts [lreplace $lacts $p $p]
	incr p 1
	set lacts [linsert $lacts $p $act]
    }
    set [set node](actions) [concat $lacts]
    redraw_nodes
}

proc nodemenu-ActionPaste {} {
    global canvas tree actionpathkeep
    if {$actionpathkeep == ""} {return}

    set path [GetPath $canvas]
    if {$path == ""} {return}
    set node [getnode $path]
    global $node

    set act [file tail $actionpathkeep]
    set oldnode [file tail [file dirname $actionpathkeep]]
    global $oldnode

    # Get next free action ...
    set newact [getnewactionnumber $node]
    append [set node](actions) " action$newact"
    global [set node]action[set newact]

    global [set oldnode][set act]
    set names [array names [set oldnode][set act]]
    foreach name $names {
 	set [set node]action[set newact]($name) [set [set oldnode][set act]($name)]
    }
    set [set node]action[set newact](path) $path
    redraw_nodes
}

proc nodemenu-InsertNode {} {
    global canvas tree
    set path [GetPath $canvas]
    if {$path == ""} {return}
    set node [getnode $path]
    global $node
    set down [set [set node](down)]
    global $down
    if {$down == "nodebase"} {
	$tree rmlink $path
	set [set down](delay) 0
	set [set down](width) 0
    }
    #Add in new node between $node and $down
    set newnode [addnewnode $path $down]
    global $newnode
    #Remove $node from $down(up)
    set ups [set [set down](up)]
    set newups ""
    foreach up $ups {
	if {$up != $node} {append newups "$up "}
    }
    set [set down](up) $newups
    set [set node](down) $newnode
    set [set newnode](up) $node
    redraw_nodes
}

proc nodemenu-AddNode {} {
    global canvas tree
    set path [GetPath $canvas]
    if {$path == ""} {return}
    set node [getnode $path]
    global $node

    #Add in new node following $node
    set newnode [addnewnode $path $node]
    global $newnode

    redraw_nodes
}

proc nodemenu-DeleteNode {} {
    global canvas tree nodepathkeep
    set path [GetPath $canvas]
    if {$path == ""} {return}
    # Connect down node to any up nodes ...
    set node [getnode $path]
    global $node
    set down [set [set node](down)]
    set ups [set [set node](up)]
    foreach up $ups {
	global $up
	set [set up](down) $down
    }
    # Connect up nodes to down node ...
    global $down
    set newups ""
    set dups [set [set down](up)]
    foreach up $dups {
	if {$up != $node} {append newups "$up "}
    }
    if {$down == "nodebase"} {$tree rmlink $path}
    set [set down](up) "$newups $ups"
    zap-node $node
    redraw_nodes
}

proc nodemenu-AddHistogram1D {} {
    global canvas
    set path [GetPath $canvas]
    if {$path == ""} {return}
    set node [getnode $path]
    global $node

    set newact [addnewhistogram $path UnNamed 1]

    redraw_nodes
}

proc nodemenu-AddHistogram2D {} {
    global canvas
    set path [GetPath $canvas]
    if {$path == ""} {return}
    set node [getnode $path]
    global $node

    set newact [addnewhistogram $path UnNamed 2]

    redraw_nodes
}

proc nodemenu-AddTAC {} {
    global canvas

    set path [GetPath $canvas]
    if {$path == ""} {return}
    set node [getnode $path]
    global $node

    set newact [addnewtac $path UnNamed]

    redraw_nodes
}

proc nodemenu-AddXYmatch {} {
    global canvas

    set path [GetPath $canvas]
    if {$path == ""} {return}
    set node [getnode $path]
    global $node

    set newact [addnewxymatch $path Position]

    redraw_nodes
}

proc nodemenu-AddGate2D {} {
    global canvas

    set path [GetPath $canvas]
    if {$path == ""} {return}
    set node [getnode $path]
    global $node

    set newact [addnewgate2d $path Gate2D]

    redraw_nodes
}

proc nodemenu-AddOutput {} {
    global canvas

    set path [GetPath $canvas]
    if {$path == ""} {return}
    set node [getnode $path]
    global $node

    set newact [addnewoutput $path Output]

    redraw_nodes
}

proc nodemenu-AddFold {} {
    global canvas

    set path [GetPath $canvas]
    if {$path == ""} {return}
    set node [getnode $path]
    global $node

    set newact [addnewfold $path Fold]

    redraw_nodes
}

proc nodemenu-ShowVars {} {
    global canvas

    set path [GetPath $canvas]
    if {$path == ""} {return}
    set node [getnode $path]
    global $node

    show-node-variables $node
}

proc nodemenu-Cut {} {
    global canvas tree nodepathkeep actionpathkeep keepmode
    set path [GetPath $canvas]
    if {$path == ""} {return}

    # If previous cut/copy in action save it ...
#    if {$keepmode == "Cut"} {
#       set node [file tail $nodepathkeep]
#       move-node-chain $node nodebase
#       redraw_nodes
#    }

    set nodepathkeep $path
    set actionpathkeep ""

    set node [file tail $nodepathkeep]
    global $node
    set down [set [set node](down)]
    global $down
    set ups [set [set down](up)]
    set newups ""
    foreach up $ups {
	global $up
	if {$up != $node} {append newups "$up "}
    }
    set [set down](up) $newups
    $tree rmlink $nodepathkeep
    set keepmode "Cut"
}

proc nodemenu-Copy {} {
    global canvas nodepathkeep actionpathkeep keepmode
    set path [GetPath $canvas]
    if {$path == ""} {return}

    # If previous cut/copy in action save it ...
    if {$keepmode == "Cut"} {
       set node [file tail $nodepathkeep]
       move-node-chain $node nodebase
       redraw_nodes
    }

    set nodepathkeep $path
    set actionpathkeep ""
    set keepmode "Copy"
}

proc move-node-chain {oldnode newnode} {
# Move node chain to new position
 
    global $newnode
    append [set newnode](up) " $oldnode"
    global $oldnode 
    set [set oldnode](down) $newnode
}

proc nodemenu-Paste {} {
    global canvas nodepathkeep actionpathkeep keepmode

    if {$nodepathkeep == ""} {
	if {$actionpathkeep == ""} {return}
	nodemenu-ActionPaste
	return
    }

    set node [file tail $nodepathkeep]
    set path [GetPath $canvas]
    if {$path == ""} {return}
    global $node 
    #puts "keepmode $keepmode"
    switch $keepmode {
	Cut {
	    set newnode [getnode $path]
	    #puts "move-node-chain $node $newnode"
	    move-node-chain $node $newnode
	}
	Copy {
	    # Have to duplicate all following nodes ...
	    duplicate-nodes $nodepathkeep $path
	}
    }
    set keepmode ""
    redraw_nodes
}

proc duplicate-node {oldpath newpath} {
    set oldnode [getnode $oldpath]
    global $oldnode
    set down [getnode $newpath]
    set newnode [addnewnode $newpath $down]
    global $newnode
    #copy over details
    set items [array names $oldnode]
    foreach item $items {
       set [set newnode]($item) [set [set oldnode]($item)]
       }
    set [set newnode](path) $newpath/$newnode
    set [set newnode](down) $down
    set [set newnode](up) ""
    # Add in any actions ...
    set acts [set [set oldnode](actions)]
    foreach act $acts {
	global [set oldnode]$act
	global [set newnode]$act
	set items [array names [set oldnode]$act]
	foreach item $items {
	    set [set newnode][set act]($item) [set [set oldnode][set act]($item)]
	}
    }
    return $newpath/$newnode
}

proc duplicate-nodes {oldpath newpath} {
    set oldnode [getnode $oldpath]
    global $oldnode
    set newpath [duplicate-node $oldpath $newpath]	
    set nodes [set [set oldnode](up)]
    foreach name $nodes {
	set node $name
	duplicate-nodes $oldpath/$node $newpath
    }
}

proc nodemenu-Edit {} {
    global canvas
    #puts nodemenu-Edit
    #ShowFiles
}

# Filing ...

proc fopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	display-error-report self $m
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
	if { ! [isComment? $t] } { lappend buf $t }
    }
    close $fp
    return $buf
}

proc write-file {f buf} {
    set fp [fopen $f w+]
    foreach line $buf {
	puts $fp $line
    }
    close $fp
    file attributes $f -permissions a+r
    set-footer "Saved to $f"
}

proc getnode {path} {
    if {$path == ""} {return nodebase}
    set n [file tail $path]
    return $n
}

proc remove_nodes {} {
    global canvas tree dir nodebase

    foreach base $nodebase(up) {
	set dir /$base
	global $base
	catch {$tree rmlink $dir}
    }
}

proc redraw_nodes {} {
    global canvas tree dir nodebase
    # Work from a virtual directory structure...

    remove_nodes

    foreach base $nodebase(up) {
	set dir /$base
	global $base
	AddDir $canvas $tree {} $dir [set [file tail $dir](name)]
	set open [set [set base](open)]
	if {$open} {
	    set frame [UniqueName $canvas $dir]
	    MakeListFrame $canvas $tree $frame $dir
	}
	ListDirsRec $canvas $tree $dir
	SetBindings $canvas $tree
    }
}

proc click-zap-setup {} {
    global canvas tree dir nodebase

    foreach base $nodebase(up) {
	set dir /$base
	global $base
	catch {$tree rmlink $dir}
    }
    set nodebase(up) ""

#    [:path FRAME2].entry delete 0 end

    zap-clusters

    $canvas delete all
}

proc set-time-scale-value {t} {
    global timeranges
    set n [string length $t]
    if {$t > "1000000000"} {return "[fourth $timeranges] [string range $t 0 [expr $n - 9]]"}
    if {$t > "1000000"}  {return "[third $timeranges] [string range $t 0 [expr $n - 6]]"}
    if {$t > "1000"} {return "[second $timeranges] [string range $t 0 [expr $n - 3]]"}
    return "[first $timeranges] $t"
}

proc set-timefrom-range-value {s r} {
    global timeranges
puts "set-timefrom-range-value $s $r"
    if {$s == "[fourth $timeranges]"} {return "[set r]00000000"}
    if {$s == "[third $timeranges]"} {return "[set r]00000"}
    if {$s == "[second $timeranges]"} {return "[set r]00"}
    return "$r"
}

proc click-loadsetupfiles {loadfrom loadboth} {
    global setupfile clustersetupname nodesetupname

#    getvxiparams
    read-templates

    set setupfile [inform tdrFiles set setupfile]

    switch $loadboth {
	0 {
	    click-clusterload [set loadfrom].cdef
	    set clustersetupname $setupfile
	}
	1 {
	    click-nodeload [set loadfrom].ndef
	    set nodesetupname $setupfile
	}
	2 {
	    click-clusterload [set loadfrom].cdef
	    set clustersetupname $setupfile
	    click-nodeload [set loadfrom].ndef
	    set nodesetupname $setupfile
	}
    }

    initvxiparams

    wm iconify [self frame-name]
    return 1
}

proc click-savesetupfiles {saveto} {
    click-clustersave $saveto.cdef
    click-nodesave $saveto.ndef
}

proc click-nodeload {f} {
    global debug MAXNODES gottext
    global cluster_types cluster_names

    clear-footer
    set soap_server [inform tdrControl set soap_server]
    start $soap_server

    remove_nodes

    set buf [read-file $f]

    set i [string last "/" $f]
    if {$i < 0} {set i 0}
    set j [string last "." $f]
    if {$j < 0} {set i end} {incr j -1}
    set gottext [string range $f [expr $i +1] $j]

    set nn 1
    while {$nn < $MAXNODES} {zap-node node$nn; incr nn}

    set currentsearch 0
    set currentnode ""
    set currentnodeaction ""
    foreach l $buf {
	if {$l == ""} {continue}
	if {[first $l] == "node"} {
	    global [second $l]
	    if {$currentnode != [second $l]} {
		# First occurrence of new node name ...
		set currentnode [second $l]
		set nn [array names $currentnode]
		foreach n $nn { unset [set currentnode]($n) }
		set [set currentnode](actions) ""
		set [set currentnode](open) 0
	    }
	} {
	    if {[string range [first $l] 0 5] == "local_"} {continue}

	    if {[first $l] == "actions"} {continue}

	    if {[first $l] == "detectors"} {
		set t [second $l]
		set c [lsearch $cluster_types $t]

		if {$c == -1} {
		    set [set currentnode](detectors) $t
		} {
		    set [set currentnode](detectors) [lindex $cluster_names $c]
		}
		puts "detectors $t $c...[set [set currentnode](detectors)]"
		continue
	    }

	    if {[first $l] == "width"} {
		set n [string length [first $l]]
		set v [set-time-scale-value [string range $l [expr $n +1] end]]
		set [set currentnode](width) [string range $l [expr $n +1] end]
		set [set currentnode](width_range) [first $v]
		set [set currentnode](width_value) [second $v]
	    }
	    if {[first $l] == "delay"} {
		set n [string length [first $l]]
		set v [set-time-scale-value [string range $l [expr $n +1] end]]
		set [set currentnode](delay) [string range $l [expr $n +1] end]
		set [set currentnode](delay_range) [first $v]
		set [set currentnode](delay_value) [second $v]
	    }

	    if {[string range [first $l] 0 5] == "gate2d"} {
		set gate [second $l]

		set n [string length [first $l]]
		set rest [string range $l [expr $n +1] end]
		set n [string length [first $rest]]
		set rest [string range $rest [expr $n +1] end]

		set [set currentnodeaction](gate$gate) $rest
		if {$gate == [set [set currentnodeaction](ngates)]} {
		    make-poly-file [set currentnode] [set currentaction]
		}
	    }

	    if {[string range [first $l] 0 5] == "action"} {
		set currentaction [first $l]
		set currentnodeaction [set currentnode][set currentaction]
		global $currentnodeaction
		set [set currentnodeaction](open) 0
		set [set currentnodeaction](type) [second $l]
		set [set currentnodeaction](name) [third $l]
		set n [string length [first $l]]
		set rest [string range $l [expr $n +1] end]
		set n [string length [first $rest]]
		set rest [string range $rest [expr $n +1] end]
		set n [string length [first $rest]]
		set rest [string range $rest [expr $n +1] end]

		set aname [second $l]

		# 2nd (y) param may not be present for 1d histograms
		if {$aname == "histogram"} {
		    set [set currentnodeaction](arg4) ""
		    set [set currentnodeaction](targ4) ""
		    set [set currentnodeaction](narg4) ""
		    set [set currentnodeaction](arg5) 0
		    set [set currentnodeaction](arg6) 0
		}
		
		switch $aname {
		    gate2d {
			set i 1
			foreach item $rest {
			    set [set currentnodeaction](arg$i) $item
			    incr i
			    if {$i == 7} {break}
			}
			set n [string first coords $rest]
			if {$n > 0} {
			    set n [expr $n + 7]
			    set rest [string range $rest $n end]
			    set [set currentnodeaction](ngates) 1
			    set [set currentnodeaction](gate1) $rest

			} {
			    set n [string first gates $rest]
			    if {$n == -1} {
				set [set currentnodeaction](ngates) 0
			    } {
				set n [expr $n + 6]
				set rest [string range $rest $n end]
				set ngates $rest
				set [set currentnodeaction](ngates) $ngates
			    }
			}
		    }
		    default {
			set i 1
			foreach item $rest {
			    set [set currentnodeaction](arg$i) $item
			    incr i
			}
		    }
		}
		
		# 2nd (y) param may not be present for 1d histograms
		if {$aname == "histogram"} {
		    split-histname [set currentnodeaction] arg1

		    set y [set [set currentnodeaction](arg5)]
		    if {$y > 0} {
			set [set currentnodeaction](dimension) 2

			split-histname [set currentnodeaction] arg4
		    } {
			set [set currentnodeaction](dimension) 1
		    }
		}

		append [set currentnode](actions) " [set currentaction]"
	    } {
		set n [string length [first $l]]
		set [set currentnode]([first $l]) [string range $l [expr $n +1] end]
		if {$debug} {puts "Node[set currentnode]: [first $l]: [set [set currentnode]([first $l])]"}
	    }
	}
    }

# Now modify noderefs since all nodes now available ...

    set nn 1
    while {$nn < $MAXNODES} {
	set node node$nn
        global $node
	set names [array names $node]
	if {$names == ""} {incr nn;continue}
	set acts [set [set node](actions)]
	foreach currentaction $acts {
	    set currentnodeaction [set node][set currentaction]
	    global $currentnodeaction

	    set type [set [set currentnodeaction](type)]
	    switch $type {
		gate2d {
		    set [set currentnodeaction](narg1) [replace-noderef [set [set currentnodeaction](arg1)]]
		    set [set currentnodeaction](narg2) [replace-noderef [set [set currentnodeaction](arg2)]]
		}
		tac {
		    set [set currentnodeaction](narg1) [replace-noderef [set [set currentnodeaction](arg1)]]
		    set [set currentnodeaction](narg2) [replace-noderef [set [set currentnodeaction](arg2)]]
		}
		XYmatch {
		    set [set currentnodeaction](narg1) [replace-noderef [set [set currentnodeaction](arg1)]]
		}
	    }
	}
	incr nn
    }

    redraw_nodes
}

proc click-nodesave {f} {
   global MAXNODES nodebase
    clear-footer

    set buf ""
    set nb [array names nodebase]
    lappend buf "node nodebase"
    foreach name $nb {
	if {$name == "open"} {continue}
	if {$name == "path"} {continue}
	if {[string range $name 0 5] == "width_"} {continue}
	if {[string range $name 0 5] == "delay_"} {continue}
	lappend buf "$name [set nodebase($name)]"
    }
    
    set nn 1
    while {$nn < $MAXNODES} {
	set node node$nn
        global $node
	# Output all nodes entries ...	
	set names [array names $node]
	if {$names == ""} {incr nn;continue}
	if {[lsearch $names name] == -1} {incr nn;continue}
	lappend buf ""
	lappend buf "node $node"
	foreach name $names {
	    if {$name == "open"} {continue}
	    if {$name == "path"} {continue}
	    if {$name == "xyparams"} {continue}
	    if {[string range $name 0 5] == "width_"} {continue}
	    if {[string range $name 0 5] == "delay_"} {continue}
	    if {[string range $name 0 5] == "local_"} {continue}
	    lappend buf "$name [set [set node]($name)]"
	}

	set d [set [set node](detectors)]
	set trec [set d]_template
	global $trec
	set xy [set [set trec](xymatch)]
	if {$xy == ""} {set xy "A B C D"}
	lappend buf "xyparams $xy"

	# Output all nodes actions, with args in order ...	
	foreach act [set [set node](actions)] {
	    global $node$act

	    set names [array names $node$act]
	    if {$names == ""} {break}
	    if {$act == "open"} {continue}
	    if {$act == "path"} {continue}
	    set atype [set [set node][set act](type)]
	    set abuf "$act $atype"
	    append abuf " [set [set node][set act](name)]"

	    switch $atype {
		gate2d {
		    set nargs 6
		    set i 1
		    while {$i} {
			set arg arg$i
			if {[lsearch $names $arg] != -1} {
			    append abuf " [set [set node][set act]($arg)]"
			}
			incr i
			if {$nargs < $i} {break}
		    }
		    set gg [read-poly-file [set node] [set act]]
		    set ngates [llength $gg]
		    append abuf " gates $ngates"
		    lappend buf $abuf

		    foreach l $gg {lappend buf $l}

		    set abuf ""
		}
		histogram {
		    join-histname [set node][set act] arg1
		    set i 1
		    while {$i < 4} {
			set arg arg$i
			if {[lsearch $names $arg] != -1} {
			    append abuf " [set [set node][set act]($arg)]"
			}
			incr i
		    }
		    split-histname [set node][set act] arg1

		    set twod [set [set node][set act](arg4)]
		    if {$twod != ""} {
			join-histname [set node][set act] arg4
			while {$i < 7} {
			    set arg arg$i
			    if {[lsearch $names $arg] != -1} {
				append abuf " [set [set node][set act]($arg)]"
			    }
			    incr i
			}
			split-histname [set node][set act] arg4
		    }
		}
		default {
		    set nargs [expr [llength $names] -2]
		    set i 1
		    while {$i} {
			set arg arg$i
			if {[lsearch $names $arg] != -1} {
			    append abuf " [set [set node][set act]($arg)]"
			}
			incr i
			if {$nargs < $i} {break}
		    }
		}
	    }
	    lappend buf $abuf
	}
	
	incr nn
    }
    
    write-file $f $buf
}

# Functions on node level

proc getnewnodenumber {} {
    global MAXNODES
    set nn 1
    while {$nn < $MAXNODES} {
	global node$nn
	if {[array names node$nn] == ""} {break}
	incr nn
    }

    return $nn
}

proc getnewactionnumber {n} {
    global MAXACTIONS $n

    set an 1
    while {$an < $MAXACTIONS} {
	set used 0
	foreach a [set [set n](actions)] {if {$a == "action$an"} {set used 1}}
	if {$used == 0} {break}
	incr an
    }
    return $an
}

proc getnewfilternumber {n} {
    global $n
    set names [array names $n]
    set i 1
    while {$i < 10} {
	if {[lsearch $names "filter$i"] == -1} {break}
	incr i
    }
    return $i
}

proc getnewconditionnumber {n} {
    global $n
    set names [array names $n]
    set i 1
    while {$i < 10} {
	if {[lsearch $names "condition$i"] == -1} {break}
	incr i
    }
    return $i
}

proc addnewnode {path node} {
    global MAXNODES nodebase
    set n [getnode $path]
    global $n

    set a [getnewnodenumber]
    if {$a == $MAXNODES} {
	set-footer "Too many nodes defined"
	return
    }

    #puts "New node = $a [list node$a] ($n)"
    
    global node$a
    set [list node$a](weight) 10000
    set [list node$a](down) $node
    set [list node$a](actions) ""
    set [list node$a](detectors) ""
    set [list node$a](up) ""
    set [list node$a](name) node$a
    
#    if {$path != ""} {
       set [list node$a](delay) 0
       set [list node$a](width) 0
       set [list node$a](delay_range) tk
       set [list node$a](delay_value) 0
       set [list node$a](width_range) tk
       set [list node$a](width_value) 0
#       }

    set [list node$a](path) $path/nodea$a
    set [list node$a](open) 1

    global $node
    lappend [set node](up) "node$a"

    if {$path == ""} {
       global canvas tree
       set base node$a
       global $base
       set dir /$base
       AddDir $canvas $tree {} $dir [set [file tail $dir](name)]
       }

    return node$a
}

proc addnewhistogram {path name dim} {
    global nodebase MAXACTIONS
    set n [getnode $path]
    global $n

    set act [getnewactionnumber $n]
    if {$act == $MAXACTIONS} {
	set-footer "Too many actions defined"
	return
    }

    lappend [set n](actions) "action$act"

    global [set n]action[set act]
    set [set n]action[set act](type) histogram
    set [set n]action[set act](name) [set name][set act]
    set [set n]action[set act](arg1) ""
    set [set n]action[set act](arg3) "0"
    set [set n]action[set act](arg4) ""
    set [set n]action[set act](arg6) "0"
    switch $dim {
	1 {
	    set [set n]action[set act](dimension) 1
	    set [set n]action[set act](arg2) "16384"
	    set [set n]action[set act](arg5) "0"
	}
	2 {
	    set [set n]action[set act](dimension) 2
	    set [set n]action[set act](arg2) "1024"
	    set [set n]action[set act](arg5) "1024"
	}
    }
    set [set n]action[set act](path) /$path/[set n]/action[set act]
    set [set n]action[set act](open) 0

    return action$act
}

proc addnewtac {path name} {
    global nodebase MAXACTIONS
    set n [getnode $path]
    global $n

    set act [getnewactionnumber $n]
    if {$act == $MAXACTIONS} {
	set-footer "Too many actions defined"
	return
    }

    lappend [set n](actions) "action$act"

    global [set n]action[set act]
    set [set n]action[set act](type) tac
    set [set n]action[set act](name) [set name][set act]
    set [set n]action[set act](arg1) start
    set [set n]action[set act](arg2) stop
    set [set n]action[set act](arg3) 8000
    set [set n]action[set act](arg4) 16384
    set [set n]action[set act](arg5) 0
    set [set n]action[set act](arg6) 0
    set [set n]action[set act](arg7) 16383

    set [set n]action[set act](path) /$path/[set n]/action[set act]
    set [set n]action[set act](open) 0

    return action$act
}

proc addnewgate2d {path name} {
    global nodebase MAXACTIONS
    set n [getnode $path]
    global $n

    set act [getnewactionnumber $n]
    if {$act == $MAXACTIONS} {
	set-footer "Too many actions defined"
	return
    }

    lappend [set n](actions) "action$act"

    global [set n]action[set act]
    set [set n]action[set act](type) gate2d
    set [set n]action[set act](name) [set name]_[set act]
    set [set n]action[set act](arg1) X
    set [set n]action[set act](arg2) Y
    set [set n]action[set act](arg3) 1024
    set [set n]action[set act](arg4) 4
    set [set n]action[set act](arg5) 1024
    set [set n]action[set act](arg6) 4

    set [set n]action[set act](ngates) 1
    set [set n]action[set act](gate1) "0 0 0 10 10 10 10 0"

    make-poly-file [set n] action[set act]

    set [set n]action[set act](path) /$path/[set n]/action[set act]
    set [set n]action[set act](open) 0

    return action$act
}

proc getXnamefromdetector {name} {

    set type [cluster-type-from-name $name]
    switch $type {
	mwpc    {set xname "=xpos"}
	sistrip {set xname "=xpos"} 
	planar  {set xname "=xpos"}
	default {set xname ""}
    }
    return $xname
}

proc getYnamefromdetector {name} {

    set type [cluster-type-from-name $name]
    switch $type {
	mwpc    {set yname "=ypos"}
	sistrip {set yname "=ypos"}
	planar  {set yname "=ypos"}
	default {set yname ""}
    }
    return $yname
}

proc addnewxymatch {path name} {
    global nodebase
    set n [getnode $path]
    global $n

    set act [getnewactionnumber $n]
    if {$act == 10} {
	set-footer "Too many actions defined"
	return
    }

    set down [set [set n](down)]
    global $down 
    set name1 [set [set down](detectors)]

    set xname1 [getXnamefromdetector $name1]
    if {$xname1 == ""} {
	set-footer "Not valid positional cluster $name1"
	return ""
    }
    set yname1 [getYnamefromdetector $name1]
    if {$yname1 == ""} {
	set-footer "Not valid positional cluster $name1"
	return ""
    }

    set name2 [set [set n](detectors)]

    set xname2 [getXnamefromdetector $name2]
    if {$xname2 == ""} {
	set-footer "Not valid positional cluster $name2"
	return ""
    }
    set yname2 [getYnamefromdetector $name2]
    if {$yname2 == ""} {
	set-footer "Not valid positional cluster $name2"
	return ""
    }

    global [set n]action[set act]
    set [set n]action[set act](type) XYmatch
    set [set n]action[set act](name) $name

    set [set n]action[set act](arg1) "=[set down]"
    set [set n]action[set act](narg1) $name1

    set [set n]action[set act](arg2) "=[set n]"
    set [set n]action[set act](narg2) $name2

    lappend [set n](actions) "action$act"

    set [set n]action[set act](arg3) 1
    set [set n]action[set act](arg4) 1

    set [set n]action[set act](path) /$path/[set n]/action[set act]
    set [set n]action[set act](open) 0

    return action$act
}

proc addnewoutput {path name} {
    global nodebase outputstyles
    set n [getnode $path]
    global $n

    set act [getnewactionnumber $n]
    if {$act == 10} {
	set-footer "Too many actions defined"
	return
    }

    lappend [set n](actions) "action$act"

    global [set n]action[set act]
    set [set n]action[set act](type) output
    set [set n]action[set act](name) $name
    set [set n]action[set act](arg1) 1
    set [set n]action[set act](arg2) [first $outputstyles]

    set [set n]action[set act](path) /$path/[set n]/action[set act]
    set [set n]action[set act](open) 0

    return action$act
}

proc addnewfold {path name} {
    global nodebase detector_fold
    set n [getnode $path]
    global $n

    set act [getnewactionnumber $n]
    if {$act == 10} {
	set-footer "Too many actions defined"
	return
    }

    lappend [set n](actions) "action$act"

    global [set n]action[set act]
    set [set n]action[set act](type) fold
    set [set n]action[set act](name) $name
    set [set n]action[set act](arg1) [first $detector_fold]

    set [set n]action[set act](path) /$path/[set n]/action[set act]
    set [set n]action[set act](open) 0

    return action$act
}

proc addnewfilter {path node} {
    global nodebase
    global $node

    set filt [getnewfilternumber $node]
    if {$filt == 10} {
	set-footer "Too many filters defined"
	return
    }

    set [set node](filter$filt) ""
}

proc addnewcondition {path node} {
    global nodebase
    global $node

    set cond [getnewconditionnumber $node]
    if {$cond == 10} {
	set-footer "Too many conditions defined"
	return
    }

    set [set node](condition$cond) ""
}

proc click-add-node {p m v} {
    global canvas tree nodebase
    clear-footer

    if {$m == "NewSearch"} {set path ""} {
    set path [GetPath $canvas]
    if {$path == ""} {return}
    }

    set n [getnode $path]
    add-node $path $n $m
    redraw_nodes
}

proc add-node {path node type} {
    global $node

    #puts "Add $type node to $node"

    switch $m {
	NewSearch {
	    addnewnode $path nodebase
	}
	NewNode {
	    addnewnode $path $n
	}
	Histogram1d {
	    addnewhistogram $path Hist 1
	}
	Histogram2d {
	    addnewhistogram $path Hist 2
	}
	TAC {
	    addnewtac $path TAC
	}
	XYmatch {
	    addnewxymatch $path XYmatch
	}
	Gate2D {
	    addnewgate2d $path Gate2D
	}
	Output {
	    addnewoutput $path Output
	}
	Fold {
	    addnewfold $path Fold
	}
	Condition {
	    addnewcondition $path $n
	}
	Filter {
	    addnewfilter $path $n
	}
    }

}

proc click-delete-node {} {
    global canvas tree nodebase
    clear-footer
    set path [GetPath $canvas]
    if {$path == ""} {return}

    set n [getnode $path]
# Node or action ...
    if {![string compare -length 6 $n "action"]} {
	set nn [string last "/" $path]
	set nodepath [string range $path 0 [expr $nn -1]]
	set node [getnode $nodepath]
        global $node
# Now remove nodes action entries ...
	set ll ""
	foreach v [set [set node](actions)] {
	    if {$v != $n} {lappend ll $v}
	}    
	set [set node](actions) $ll
    } {
	global $n
	if { [set [set n](up)] == ""} {
	    $tree rmlink $path
	} {
	    set-footer "Sorry, Cannot delete unless an end node"
	    return
	}
	
	set down [set [set n](down)]
	global $down
	# Remove up link in next level... 
	set ll ""
	foreach v [set [set down](up)] {
	    if {$v != $n} {lappend ll $v}
	}    
	set [set down](up) $ll
	# Remove all this nodes entries ...
	foreach v [array names $n] {unset [set n]($v)}
    }
    redraw_nodes
}

proc click-print-canvas {} {
    global canvas
    clear-footer
    raise .
    set sr [$canvas cget -scrollregion]
    set x [first $sr]
    set y [second $sr]
    set wid [expr [third $sr] - [first $sr]]
    set hei [expr [fourth $sr] - [second $sr]]
    $canvas postscript -x $x -y $y -width $wid -height $hei -pagewidth 5i -file /tmp/TDR_canvas.ps
    print-postscript /tmp/TDR_canvas.ps
}

proc click-help {} {
    global basedir
    clear-footer
    set f [list "" $basedir/manual/tdr.html]
    inform help html_show $f
}

# If an actions hit - show it ...

proc ShowActions {canvas tree} {
    set dir [GetPath $canvas]

    set action [file tail $dir]

   if {[string compare -length 6 $action "action"]} {
	return
    }

    set d [file dirname $dir]
    set node [file tail $d]
    global [set node][set action]
    global [set node]

    set type [set [set node][set action](type)]
    #puts "ShowActions node $node action $action"
    switch $type {
	histogram {
	    set name [set [set node](name)].[set [set node][set action](name)]
	    #puts "ShowActions node $node action $action ... $name"
	    inform sd show $name
	}
	tac {
	    set name [set [set node](name)].[set [set node][set action](name)]
	    #puts "ShowActions node $node action $action ... $name"
	    inform sd show $name
	}
	output {
	    inform tape start
	}
	default {
	}
    }
}


# Display the file names in the selected directory in
# a scrolling list beneath the node

proc ShowFiles {canvas tree} {
    set dir [GetPath $canvas]
    set frame [UniqueName $canvas $dir]
    # create the frame and list if not already there
    if {![llength [$canvas find withtag $dir:list]]} {
        RemoveListFrame $canvas $tree $frame $dir
        MakeListFrame $canvas $tree $frame $dir
    } else {
        RemoveListFrame $canvas $tree $frame $dir
    }
}


# generates a unique name for a widget from the given
# directory name by replacing the '.'s with '_'s

proc UniqueName {canvas dir} {
    global dirtree
    if {![catch {set path $dirtree($canvas.$dir)}]} {
	return $path
    }
    if {[catch {incr dirtree(listboxcnt)}]} {
	set dirtree(listboxcnt) 0
    } 
    set dirtree($canvas.$dir) $canvas.list$dirtree(listboxcnt)
    return $dirtree($canvas.$dir)
}


# Define the input fields in the list frame

proc keyvalue-validate-name {v} {
    return 1
}

proc keyvalue-validate-range {v} {
    if {$v < 0} {return 0}
    if {$v > 16384} {return 0}
    return 1
}

proc keyvalue-validate-scale {v} {
    if {$v < 0} {return 0}
    if {$v > 16} {return 0}
    return 1
}

proc keyvalue-validate-weight {v} {
#    if {$v < 0} return 0
#    if {$v > 1000000} return 0
    return 1
}

proc keyvalue-validate-delay {v} {
#    if {$v < -1000} return 0
#    if {$v > 2000000000} return 0
    return 1
}

proc keyvalue-validate-width {v} {
#    if {$v < 0} return 0
#    if {$v > 2000000000} return 0
    return 1
}

proc keyvalue-validate-detectors {v} {
    return 1
}

proc keyvalue-update {frame wname node} {
    global canvas $node
    set d [$frame.$wname.text get]

    # Validate input...

    if {[info procs keyvalue-validate-$wname] != ""} {
	set ret [keyvalue-validate-$wname $d]
	if {!$ret} {
	    set-footer "$wname modification invalid"
	    return
	}
    } {
	set ret 0
	if {[string range $wname 0 2] != "arg"} {
	    set-footer "No validate handler for $wname"
	}
    }

# Now carry on ...

    $frame.$wname.text delete 0 end
    $frame.$wname.text insert 0 $d
    set [set node]($wname) $d

# If changing "name" also change node label ...

    if {$wname == "name"} {
	set n $node
	set i [string first a $node]
	if {$i > 0} {
	    set n [string range $node 0 [expr $i -1]]
	    set act [string range $node $i end]
	    global $n$act
	    set dir [set [set n][set act](path)]
	} {
	    global $n	
	    set dir [set [set node](path)]
	}
	$canvas itemconfigure $dir:text -text $d
    }
    if {$ret} {set-footer "$wname modified"}
}

proc keyvaluedeaf {frame wname name node} {
    global $node
    set bgcolour honeydew
    frame $frame.$wname -background white
    label $frame.$wname.mess -width 10 -text $name -anchor w \
	-background $bgcolour
    pack $frame.$wname.mess -side left

    label $frame.$wname.text -width 20 -text [set [set node]($wname)] -anchor w \
	-background $bgcolour
    pack $frame.$wname.text -side right

    pack $frame.$wname -side top
}

proc keyvalue {frame wname name node} {
    global $node
    frame $frame.$wname -background white
    label $frame.$wname.mess -width 10 -text $name -anchor w \
	-background white
    pack $frame.$wname.mess -side left

    entry $frame.$wname.text -width 20 -background MintCream
    $frame.$wname.text insert 0 [set [set node]($wname)]
    pack $frame.$wname.text -side right

    bind  $frame.$wname.text <Leave> "keyvalue-update $frame $wname $node"

    pack $frame.$wname -side top
}

proc click-show {f} {
    global specbase tdrEBResource
    inform sd show /$tdrEBResource/$specbase/$f
}

proc click-mapshow {f} {
    global specbase
    inform sd show $specbase/maps/$f
}

# Polygon gate procedures ...

set defdir [file join $env(HOME) .eg-polyarrays]

proc make-poly-file {node action} {
    global defdir $node $node$action
 
# If $defdir does not exist - create it

    if {![file exists $defdir]} {
       set z [catch {:file mkdir $defdir}]
       if {$z} {
          set-footer "Cannot create polyarrays directory"
          return
       }
   }

    set gates ""
    set ngates [set [set node][set action](ngates)]
    for {set g 1} {$g <= $ngates} {incr g} {
	set gs [set [set node][set action](gate$g)]
	set gate ""
	for {set i 0} {$i < 64} {} {
	    set gx [lindex $gs $i]
	    if {$gx == ""} {break}
	    incr i
	    set gy [lindex $gs $i]
	    if {$gx == ""} {break}
	    incr i
	    lappend gate [list $gx $gy]
	}
	lappend gates [list $gate]
    }

    set name1 [set [set node](name)]
    set name2 [set [set node][set action](name)]
    set file [set name1].[set name2]
    write-file [file join $defdir $file] $gates
}

proc read-poly-file {node action} {
    global defdir $node $node$action

    set name1 [set [set node](name)]
    set name2 [set [set node][set action](name)]
    set file [set name1].[set name2]

    set buf [read-file [file join $defdir $file]]
    set buf [concat $buf]
    set abuf ""
    set ngates 0
    foreach l $buf {
	foreach m $l {
	    incr ngates
	    set m2 [concat $m]
	    set gg ""
	    foreach g $m2 {append gg " $g"}
	    lappend abuf "gate2d $ngates $gg"
	}
    }
return $abuf
}

proc click-polyshow {f} {
    global defdir
    set z [catch {:file mkdir $defdir}]
    inform sd load-polyarray-file $defdir $f
}

proc click-polysave {f} {
    global defdir
   inform sd save-polyarray-file $defdir $f pdb
}

# Node/Action display procedures ...

proc histt-compat {frme wname node action} {
    global frame $node $action env
    set fkeep $frame
    set frame $frme

    set w [set action]$wname
    :frame [self frame-name].$w :toplevel F :show-footer F :win-ctrl N
#    [:path [self frame-name].$w] configure -background white

    :panel HISTT  :layout R :padx 0 :pady 0
      :layout H;

    set name [set [set action](name)]
    :text TITLE :label "Hist" :value $name

    bind [:path TITLE].entry <Leave> "keyvalue-update $frame $wname $node"

    set nodename [set [set node](name)]
    :button SHOW :label "@$env(ICONHOME)/spectrum" :command "click-show $nodename.$name"
    :show T
    set frame $fkeep
}

proc menu-update {wname node name d} {
    global canvas $node

    if {[set [set node]($name)] == $d} {clear-footer; return}

    #puts "menu-update $wname $node $name $d"

# Validate input...

    if {[info procs keyvalue-validate-$name] != ""} {
        set ret [keyvalue-validate-$name $d]
        if {!$ret} {
            set-footer "$name modification invalid"
            return
        }
    } {
        set ret 0
        set-footer "No validate handler for $name"
    }

# Now set new value ...

    set [set node]($name) $d

# If changing "name" also change node label ...

    if {$name == "name"} {
        set n $node
        set i [string first a $node]
        if {$i > 0} {
            set n [string range $node 0 [expr $i -1]]
            set act [string range $node $i end]
            global $n$act
            set dir [set [set n][set act](path)]
        } {
            global $n
            set dir [set [set node](path)]
        }
        $canvas itemconfigure $dir:text -text $d
    }

    if {$ret} {set-footer "$name modified to $d"}
}

proc menu-timeentryupdate {wname node name d} {
    global canvas $node

#    puts "menu-timeentryupdate $wname $node $name $d"

    if {[set [set node]([set name]_value)] == $d} {clear-footer; return}

# Validate input...

    if {[info procs keyvalue-validate-$name] != ""} {
        set ret [keyvalue-validate-$name $d]
        if {!$ret} {
            set-footer "$name modification invalid"
            return
        }
    } {
        set ret 0
        set-footer "No validate handler for $name"
    }

# Now set new value ...

    set [set node]([set name]_value) $d
    set tr [set [set node]([set name]_range)]
    set [set node]([set name]) [set-timefrom-range-value $tr $d]

    if {$ret} {set-footer "$name modified to $d"}
}

proc menu-timeupdate {wname node name d} {
    global canvas $node timeranges timescales

#puts "menu-timeupdate $wname $node $name $d"

    set v [lsearch $timeranges $d]
    if {[set [set node]([set name]_range)] == $v} {clear-footer; return}

# Validate input...

    if {[info procs keyvalue-validate-$name] != ""} {
	set ret [keyvalue-validate-$name $d]
	if {!$ret} {
	    set-footer "$name modification invalid"
	    return
	}
    } {
	set ret 0
	set-footer "No validate handler for $name"
    }

# Now set new value ...

    set t [set [set node]([set name]_range)]
    set tv [set [set node]([set name]_value)]
    set tr [lindex $timescales $v]
    set [set node]([set name]) [set-timefrom-range-value $d $tv]
    set [set node]([set name]_value) $tv 
    set [set node]([set name]_range) $d

    if {$ret} {set-footer "$name modified to $d"}
}

proc menuhistaddr-update {wname node name d} {
    global $node

    set current [set [set node](t$name)]
    
    if {$current == $d} {clear-footer; return}

    if {$d == "p"} {set d ""}
#    set n [lsearch "p a g t" $d]
#    if {$n > 0} {set d [lindex "& ! @" $n]} {set d ""}
    
    set [set node](t$name) $d
    
    #puts "menuhistaddr-update $wname $node $name $d"
}

proc menuhistname-update {win node name n1 n2} {
    global $node

    set d [set n1]:[set n2]
    set current [set [set node]($name)]
    if {$d == $current} {clear-footer; return}
    set cn [replace-noderef $current]
    if {$d == $cn} {clear-footer; return}
    
    # not same so check if node is specified ...
    
    set i [string first ":" $d]
    if {$i != -1} {
	set sm [string range $d $i end]
	incr i -1
	if {[string range $d 0 4] == "=node"} {
	    set othernode [string range $d 1 $i]
	    global $othernode
	    set refname "=$othernode[set sm]"
	    set newname [replace-noderef $d]
	} {
	    set sn [string range $d 0 $i]
	    set newnode [get-node-from-name $sn]
	    set refname [string replace $d 0 $i "=$newnode"]
	    set newname $d
	}
    } {
	set refname $d
	set newname $d
    }
    
    # Now set new value ...
    
    set [set node]($name) $refname
    set [set node](n[set name]) $newname
    
    #puts "menuhistname-update $win $node $name $newname"
}

proc menunodename-update {wname node name d} {
    global $node

    set current [set [set node]($name)]
    
    if {[string range $d 0 4] == "=node"} {
	if {$current == $d} {clear-footer; return}
	set othernode [string range $d 1 end]
	global $othernode
	set refname "=$othernode"
	set newname [replace-noderef $d]
    } {
	set othernode [get-node-from-name $d]
	if {$othernode == ""} {
	    set refname "=$node"
	    set newname $d
	} {
	    if {$current == "=$othernode"} {clear-footer; return}
	    global $othernode
	    set newname [set [set othernode](name)]
	    set refname "=$othernode"
	}
    }
    
    # Now set new value ...
    
    set [set node]($name) $refname
    set [set node](n[set name]) $newname
    
    #puts "menunodename-update $wname $node $name $d"
}

proc menudetectorname-update {wname node name type} {
    global $node cluster_names

    set current [set [set node]([set name])]
    
    if {$current == $type} {clear-footer; return}
	
    set [set node]([set name]) $type
    
    puts "menudetectorname-update $wname $node $name $type"

    # Initialise if new cluster ...

    if {[lsearch -exact $cluster_names $type] == -1} {
	add-cluster $type
	cluster-draw-window node $node
    }
}

proc menubutton-menu {win name st proc} {
    global menuoptions
    eval menu $win.[set name]menu $menuoptions
    set ii 0
    set jj 0
    foreach s $st {
	if {$jj == 10} then {
	    $win.[set name]menu add command -label "$s" -columnbreak 1 \
                -command "menu-update $proc \"$s\""
             set jj 0
          } else {
             $win.[set name]menu add command -label "$s"  \
                -command "menu-update $proc \"$s\""
          }
          incr ii
          incr jj
        }
}

proc menubutton-timemenu {win name st proc} {
    global menuoptions
    eval menu $win.[set name]menu $menuoptions
    set ii 0
    set jj 0
    foreach s $st {
        if {$jj == 10} then {
            $win.[set name]menu add command -label "$s" -columnbreak 1 \
                -command "menu-timeupdate $proc \"$s\""
             set jj 0
	} else {
             $win.[set name]menu add command -label "$s"  \
                -command "menu-timeupdate $proc \"$s\""
	}
          incr ii
          incr jj
    }
}

proc menubutton-detectormenu {win name proc} {
    global templates menuoptions
    eval menu $win.[set name]menu $menuoptions

#    set nt [get-detector-list ""]
    set nt [get-template-list]
    set ii 0
    set jj 0
    foreach s $nt {
	global $s
	if {$jj == 10} then {
	    $win.[set name]menu add command -label "$s" -columnbreak 1 \
                -command "menudetectorname-update $proc \"$s\""
	    set jj 0
	} else {
	    $win.[set name]menu add command -label "$s" \
                -command "menudetectorname-update $proc \"$s\""
	}
	incr ii
	incr jj
    }
}

proc menubutton-nodemenu {win name node proc} {
    global menuoptions
    eval menu $win.[set name]menu $menuoptions

    set nt [get-down-node-list $node]
    set ii 0
    set jj 0
    foreach n $nt {
	global $n
	set s [set [set n](name)]
	if {$jj == 10} then {
	    $win.[set name]menu add command -label "$s" -columnbreak 1 \
                -command "menunodename-update $proc \"$s\""
	    set jj 0
	} else {
	    $win.[set name]menu add command -label "$s" \
                -command "menunodename-update $proc \"$s\""
	}
	incr ii
	incr jj
    }
}

proc menubutton-histaddrmenu {win name nt proc} {
    global menuoptions
    eval menu $win.[set name]menu $menuoptions

    set ii 0
    set jj 0
    foreach s $nt {
	global $s
	if {$jj == 10} then {
	    $win.[set name]menu add command -label "$s" -columnbreak 1 \
                -command "menuhistaddr-update $proc \"$s\""
	    set jj 0
	} else {
	    $win.[set name]menu add command -label "$s" \
                -command "menuhistaddr-update $proc \"$s\""
	}
	incr ii
	incr jj
    }
}

proc menubutton-histmenu {win name proc} {
    global menuoptions
    eval menu $win.[set name]menu $menuoptions

    set ii 0
    set jj 0
    set nn [second $proc]
    set i [string first action $nn]
    set n [string range $nn 0 [incr i -1]]
    global $n
    set s [set [set n](name)]
    if {$jj == 10} then {
	$win.[set name]menu add command -label "$s" -columnbreak 1 \
	    -command "menu[set name]name-update $proc \"$s\""
	set jj 0
    } else {
	$win.[set name]menu add cascade -label "$s" \
	    -menu $win.[set name]menu.menu$ii
	
	eval menu $win.[set name]menu.menu$ii $menuoptions
puts "menubutton-histmenu: $n"
	set pt [get-histparam-list $n]
	foreach p $pt {
	    $win.[set name]menu.menu$ii add command -label "$p"  \
		-command "menu[set name]name-update $proc \"$s\" \"$p\""
	    
	}
    }
    incr ii
    incr jj
}

proc menubutton-histmenu {win name proc} {
    global menuoptions
    eval menu $win.[set name]menu $menuoptions

    set nt [get-node-list ""]
    set ii 0
    set jj 0
    foreach n $nt {
	global $n
	set s [set [set n](name)]
	if {$jj == 10} then {
	    $win.[set name]menu add command -label "$s" -columnbreak 1 \
                -command "menu[set name]name-update $proc \"$s\""
             set jj 0
          } else {
             $win.[set name]menu add cascade -label "$s" \
		 -menu $win.[set name]menu.menu$ii

	      eval menu $win.[set name]menu.menu$ii $menuoptions

	      set pt [get-histparam-list $n]
	      foreach p $pt {
		  $win.[set name]menu.menu$ii add command -label "$p"  \
		      -command "menu[set name]name-update $proc \"$s\" \"$p\""

	      }
	  }
	incr ii
	incr jj
    }
}

proc getnoderef {sn} {
    set i [string first ":" $sn]
    if {$i != -1} {
	incr i -1
	set s [string range $sn 0 $i]
	if {[string range $s 0 4] == "=node"} {
	    return $s
	} {
	    return ""
	}
    }
    return ""
}

proc isnoderef {sn} {
    set i [string first ":" $sn]
    if {$i != -1} {
	incr i -1
	set s [string range $sn 1 $i]
	if {[string range $sn 0 4] == "=node"} {
	    return 1
	} {
	    return 0
	}
    }
    return 0
}

proc replace-noderef {sn} {
    set i [string first ":" $sn]
    if {$i != -1} {
	incr i -1
	if {[string range $sn 0 4] == "=node"} {
	    set node [string range $sn 1 $i]
	    global $node
	    set name [set [set node](name)]
	    set sn [string replace $sn 0 $i $name]
	}
    } {
	if {[string range $sn 0 4] == "=node"} {
	    set node [string range $sn 1 end]
	    global $node
	    set sn [set [set node](name)]
	}
    }
    return $sn
}

proc split-histname {act arg} {
    global $act
    set name [set [set act]($arg)]
    set addr [string range $name 0 0]
    switch $addr {
	& -
	! -
	@ {
	    set n [lsearch "& ! @" $addr]
	    set addr [lindex "a g t" $n]
	    set name [string range $name 1 end]
	}
	default {
	    set addr ""
	}
    }
    set [set act](t$arg) $addr
    set [set act](n$arg) [replace-noderef $name]
}

proc join-histname {act arg} {
    global $act

    set addr [set [set act](t$arg)]
    switch $addr {
	p {set a ""}
	a {set a "&"}
	g {set a "!"}
	t {set a "@"}
	default {set a ""}
    }

    set name [set a][set [set act](n$arg)]
    set [set act]($arg) $name
}

proc drawlabel {win width text} {
    label $win -width $width -text $text -anchor w -background wheat
    pack $win -side left
}

proc drawentry {win width wname act arg} {
    global $act
    entry $win -width $width -background MintCream
    $win insert 0 [set [set act]($arg)]
    pack $win -side left
    bind  $win <Leave> "menu-update $wname $act $arg \[$win get\]"
}

proc drawmessage {win width text} {
    message $win -width $width -text $text -padx 0 -background wheat
    pack $win -side left
}

proc drawsetup {win width node} {
    global $node
    button $win -width $width -text "Setup" -command "cluster-draw-window node $node" -padx 1 -background MintCream -relief raised
    pack $win -side right
}

proc drawspecshow {win node act map} {
    global $node $act
    set nodename [set [set node](name)]
    set name [set [set act](name)]
    if {$map == "map"} {set d "map"} {set d ""}
    button $win -height 16 -image Spectrum -command "click-[set d]show $nodename.$name" -padx 1 -background pink
    pack $win -side right
}

proc drawgate2dshow {win node act} {
    global $node $act
    set nodename [set [set node](name)]
    set name [set [set act](name)]
    button $win -height 16 -width 8 -text "Show" -command "click-polyshow $nodename.$name" -padx 1 -background pink
    pack $win -side right
}

proc drawgate2dsave {win node act} {
    global $node $act
    set nodename [set [set node](name)]
    set name [set [set act](name)]
    button $win -height 16 -width 8 -text "Save" -command "click-polysave $nodename.$name" -padx 1 -background pink
    pack $win -side right
}

proc drawnodeentry {win w wname node act arg} {
    global menubuttonoptions $node $act

    eval menubutton [set win] -menu [set win].nodemenu -width [incr w -1] \
	-textvariable [set act](n[set arg]) $menubuttonoptions
    menubutton-nodemenu $win node $node "$win $act $arg"
    pack $win -side left
}

proc drawdetectorentry {win w wname node arg} {
    global menubuttonoptions $node MAXNODES

    eval menubutton [set win] -menu [set win].detmenu -width [incr w -1] \
	-textvariable [set node]([set arg]) $menubuttonoptions
    
    menubutton-detectormenu $win det "$win $node $arg"

    pack $win -side left
}

proc drawhistentry {win w wname node act arg} {
    global menubuttonoptions $node $act MAXNODES

    eval menubutton [set win]a -menu [set win]a.addrmenu -width 1 \
	-textvariable [set act](t$arg) $menubuttonoptions
    menubutton-histaddrmenu [set win]a addr  "p a g t" "[set win]a $act $arg"
    pack [set win]a -side left -expand no

    eval menubutton $win -menu [set win].histmenu -width [incr w -3] \
	-textvariable [set act](n[set arg]) $menubuttonoptions
    menubutton-histmenu $win hist "$win $act $arg"
    pack $win -side left
}

proc drawrange {win w wname act arg} {
    global menubuttonoptions histranges $act
    eval menubutton $win -menu $win.rangemenu -width $w -textvariable [set act]($arg) $menubuttonoptions
    menubutton-menu $win range $histranges "$wname $act $arg"
    pack $win -side left
}

proc drawstrings {win w wname act arg vstring} {
    global menubuttonoptions $act
    eval menubutton $win -menu $win.stringsmenu -width $w \
	-textvariable [set act]($arg) $menubuttonoptions
    menubutton-menu $win addr $vstring "$win $act $arg"
    pack $win -side left -expand no
}

proc drawvalues {win w wname act arg vstring} {
    global menubuttonoptions $act
    eval menubutton $win -menu $win.valuesmenu -width $w -textvariable [set act]($arg) $menubuttonoptions
    menubutton-menu $win values $vstring "$wname $act $arg"
    pack $win -side left
}

proc drawscale {win w wname act arg} {
    global menubuttonoptions histscales $act
    eval menubutton $win -menu $win.scalemenu -width $w -textvariable [set act]($arg) $menubuttonoptions
    menubutton-menu $win scale $histscales "$wname $act $arg"
    pack $win -side right -expand no
}

proc drawdelay {win w wname act arg} {
    global menubuttonoptions tacdelays $act
    eval menubutton $win -menu $win.rangemenu -width $w -textvariable [set act]($arg) $menubuttonoptions
    menubutton-menu $win range $tacdelays "$wname $act $arg"
    pack $win -side left
}

proc drawgate {win w wname act arg} {
    global $act
    entry $win -width $w -background MintCream
    $win insert 0 [set [set act]($arg)]
    pack $win -side left
    bind $win <Leave> "menu-update $wname $act $arg \[$win get\]"
}

# Routines for drawing action/node item boxes ...

proc draw-histt {frame wname node action} {
    global $action
    set w [set action]t
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess 6 "Hist"
    drawentry $frame.$w.text 15 $wname $action name
    drawspecshow $frame.$w.show $node $action shm

    pack $frame.$w -side top -anchor w
}

proc draw-histx {frame wname node action} {
    global $action
    set w [set action]x
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess 6 "Xparam"
    drawhistentry $frame.$w.text 15 $wname $node $action arg1
    drawrange $frame.$w.range 4 $wname $action arg2
    drawmessage $frame.$w.scalemess 1 "/"
    drawscale $frame.$w.scale 1 $wname $action arg3

    pack $frame.$w -side top -anchor w
}

proc draw-histy {frame wname node action} {
    global $action
    set w [set action]y
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess 6 "Yparam"
    drawhistentry $frame.$w.text 15 $wname $node $action arg4
    drawrange $frame.$w.range 4 $wname $action arg5
    drawmessage $frame.$w.scalemess 1 "/"
    drawscale $frame.$w.scale 1 $wname $action arg6

    pack $frame.$w -side top -anchor w
}

proc draw-tact {frame wname node action} {
    global $node $action
    set w [set action]t
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess 6 "Tac"
    drawentry $frame.$w.text 15 $wname $action name
    drawspecshow $frame.$w.show $node $action shm

    pack $frame.$w -side top -anchor w
}

proc draw-tac1 {frame wname node action} {
   global $action
    set w [set action]x
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess 6 "Start"
    drawhistentry $frame.$w.text 15 $wname $node $action arg1
    drawrange $frame.$w.range 4 $wname $action arg4
    drawmessage $frame.$w.scalemess 1 "/"
    drawscale $frame.$w.scale 1 $wname $action arg5

    pack $frame.$w -side top -anchor w
}

proc draw-tac2 {frame wname node action} {
    global $action
    set w [set action]y
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess 6 "Stop"
    drawhistentry $frame.$w.text 15 $wname $node $action arg2
    drawlabel $frame.$w.delaymess 3 "Dly"
    drawdelay $frame.$w.delay 4 $wname $action arg3

    pack $frame.$w -side top -anchor w
}

proc draw-tacg {frame wname action} {
    global $action
    set w [set action]g
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess1 6 "Gate"
    drawgate $frame.$w.gate1 10 $wname $action arg6
    drawlabel $frame.$w.mess2 3 " to "
    drawgate $frame.$w.gate2 10 $wname $action arg7

    pack $frame.$w -side top -anchor w
}

proc draw-gate2dt {frame wname node action} {
    global $node $action
    set w [set action]t
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess 6 "Gate2D"
    drawentry $frame.$w.text 15 $wname $action name
    drawspecshow $frame.$w.show $node $action map

    pack $frame.$w -side top -anchor w
}

proc draw-gate2dx {frame wname node action} {
    global $action
    set w [set action]x
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess 6 "Xparam"
    drawhistentry $frame.$w.text 14 $wname $node $action arg1
    drawrange $frame.$w.range 5 $wname $action arg3
    drawmessage $frame.$w.scalemess 1 "/"
    drawscale $frame.$w.scale 1 $wname $action arg4

    pack $frame.$w -side top -anchor w
}

proc draw-gate2dy {frame wname node action} {
    global $action
    set w [set action]y
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess 6 "Yparam"
    drawhistentry $frame.$w.text 14 $wname $node $action arg2
    drawrange $frame.$w.range 5 $wname $action arg5
    drawmessage $frame.$w.scalemess 1 "/"
    drawscale $frame.$w.scale 1 $wname $action arg6

    pack $frame.$w -side top -anchor w
}

proc draw-gate2dg {frame wname action} {
    global $action
    set w [set action]g
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess 6 "Coords"
    drawentry $frame.$w.text 26 $wname $action arg7

    pack $frame.$w -side top -anchor w
}

proc draw-gate2dp {frame wname node action} {
    global $action
    set w [set action]p
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess1 6 "Gates"
    drawlabel $frame.$w.mess2 7 "          "
    drawgate2dshow $frame.$w.show $node $action
    drawgate2dsave $frame.$w.save $node $action

    pack $frame.$w -side top -anchor w
}

proc draw-post {frame wname node action} {
    global $node $action
    set w [set action]t
    frame $frame.$w -background wheat -height 25

    drawlabel $frame.$w.mess 6 "Match"
    drawentry $frame.$w.text 15 $wname $action name
    drawmessage $frame.$w.fillmess 25m "                "

    pack $frame.$w -side top -anchor w
}

proc draw-pos1 {frame wname node action} {
    global $action
    set w [set action]1
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess1 6 "With"
    drawnodeentry $frame.$w.text 10 $wname $node $action arg1
    drawlabel $frame.$w.mess2 2 "+-"
    drawentry $frame.$w.xmatch 5 $wname $action arg3
    drawentry $frame.$w.ymatch 5 $wname $action arg4

    pack $frame.$w -side top -anchor w
}

proc draw-outputt {frame wname node action} {
    global $action outputstyles
    set w [set action]t
    frame $frame.$w -background wheat -height 25

    drawlabel $frame.$w.mess 8 "Output"
    drawvalues $frame.$w.styles 22 $wname $action arg2 $outputstyles

    pack $frame.$w -side top -anchor w
}

proc draw-output1 {frame wname action} {
    global $action outputchannels
    set w [set action]1
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess 8 "Channel"
    drawvalues $frame.$w.channel 1 $wname $action arg1 $outputchannels
    drawmessage $frame.$w.fillmess 70m "                                       "

    pack $frame.$w -side top -anchor w
}

proc draw-foldt {frame wname node action} {
    global $action detector_fold
    set w [set action]1
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess 8 "Fold"
    drawvalues $frame.$w.channel 1 $wname $action arg1 $detector_fold
    drawmessage $frame.$w.fillmess 70m "                                       "

    pack $frame.$w -side top -anchor w
}
proc draw-foldt {frame wname node action} {}

proc draw-nodet {frame wname node} {
    global $node
    set w [set node]t
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess 7 "Node"
    drawentry $frame.$w.text 15 $wname $node name
    drawmessage $frame.$w.fillmess 3m "  "
    drawsetup $frame.$w.setup 6 $node

    pack $frame.$w -side top -anchor w
}

proc draw-node1 {frame wname node} {
    global $node
    set w [set node]1
    frame $frame.$w -background wheat -height 24

    drawlabel $frame.$w.mess1 7 "Detector"
    drawdetectorentry $frame.$w.det 24 $wname $node detectors
#    drawlabel $frame.$w.mess2 6 "Weight"
#    drawentry $frame.$w.weight 6 $wname $node weight

    pack $frame.$w -side top -anchor w
}

proc drawtimeentry {win width wname act arg} {
    global menubuttonoptions $act MAXNODES timeranges

    eval menubutton [set win]a -menu [set win]a.addrmenu -width 1 \
        -textvariable [set act]([set arg]_range) $menubuttonoptions
    menubutton-timemenu [set win]a addr $timeranges "[set win]a $act $arg"
    pack [set win]a -side left -expand no

    entry $win -width [incr width -2]  -background MintCream
    $win insert 0 [set [set act]([set arg]_value)]
    pack $win -side left
    bind  $win <Leave> "menu-timeentryupdate $wname $act $arg \[$win get\]"
}

proc draw-node2 {frame wname node} {
    global $node
    set w [set node]2
    frame $frame.$w -background wheat -height 20

    drawlabel $frame.$w.mess1 6 "Delay"
    drawtimeentry $frame.$w.delay 8 $wname $node delay
    drawlabel $frame.$w.mess2 5 "Width"
    drawtimeentry $frame.$w.width 8 $wname $node width

    pack $frame.$w -side top -anchor w
}


# make the list frame for displaying the files in dir

proc MakeListFrame {canvas tree frame dir} {
    global dirtree bitmaps env

    set node [file tail $dir]
    global $node
   if {[string compare -length 6 $node "action"]} {
	if {![llength [array get [set node] detectors]]} {return}
    }

    set bg [$canvas cget -bg]
    
    # make the list frame and set up resizing 
    set colour [$canvas itemconfig $dir:text -fill]
    set colour [fifth $colour]

    frame $frame -borderwidth 3 -cursor bottom_right_corner \
	-background $colour

    image create bitmap Spectrum -file $env(ICONHOME)/spectrum \
        -maskfile $env(ICONHOME)/spectrum
    image create bitmap Range -file $env(ICONHOME)/expand \
        -maskfile $env(ICONHOME)/expand
    image create bitmap Scale -file $env(ICONHOME)/times2 \
        -maskfile $env(ICONHOME)/times2

    if {![string compare -length 6 $node "action"]} {
	set d [file dirname $dir]
	set nact [file tail $d]
        global [set nact][set node]

        set a [set [list [set nact][set node](type)]]
        set nn [set [list [set nact][set node](name)]]

puts "[set [list [set nact][set node](type)]]"
	switch $a {
	    histogram {
                draw-histt $frame [set nn] [set nact] [set nact][set node]
                draw-histx $frame [set nn] [set nact] [set nact][set node]
		set dim [set [list [set nact][set node](dimension)]]
		if {$dim == 2} {
		    draw-histy $frame [set nn] [set nact] [set nact][set node]
		    set nrows 3
		} {
		    set nrows 2
		}
	    }
	    tac {
                draw-tact $frame [set nn] [set nact] [set nact][set node]
                draw-tac1 $frame [set nn] [set nact] [set nact][set node]
                draw-tac2 $frame [set nn] [set nact] [set nact][set node]
                draw-tacg $frame [set nn] [set nact][set node]
		set nrows 4
	    }
	    output {
                draw-outputt $frame [set nn] [set nact] [set nact][set node]
                draw-output1 $frame [set nn] [set nact][set node]
		set nrows 2
	    }
	    fold {
#                draw-foldt $frame [set nn] [set nact] [set nact][set node]
#                draw-fold1 $frame [set nn] [set nact][set node]
#		set nrows 1
	    }
	    XYmatch {
                draw-post $frame [set nn] [set nact] [set nact][set node]
                draw-pos1 $frame [set nn] [set nact] [set nact][set node]
		set nrows 2
	    }
	    gate2d {
                draw-gate2dt $frame [set nn] [set nact] [set nact][set node]
                draw-gate2dx $frame [set nn] [set nact] [set nact][set node]
		draw-gate2dy $frame [set nn] [set nact] [set nact][set node]
#		draw-gate2dg $frame [set nn] [set nact][set node]
		draw-gate2dp $frame [set nn] [set nact] [set nact][set node]
		set nrows 4
	    }
	    default {
		keyvaluedeaf $frame type "Action" [set nact][set node]
		keyvalue $frame name "Name" [set nact][set node]
		keyvalue $frame arg1 "Arg1" [set nact][set node]
		set nrows 3
	    }
	}
	# Set this node as open 
	set [set nact][set node](open) 1

	set h [expr $nrows * 24 + 2]
    } {
        set nn [set [list [set node](name)]]

	draw-nodet $frame [set nn] [set node]
	draw-node1 $frame [set nn] [set node]
	if {[set [set node](down)] != "nodebase"} {
	    draw-node2 $frame [set nn] [set node]
	    set nrows 3
	} {
#	    draw-node2 $frame [set nn] [set node]
#	    set nrows 3
	    set nrows 2
	}

	# Set this node as open 
	set [set node](open) 1

	set h [expr $nrows * 25 +2]
    }

# insert the list frame in the canvas
    switch $env(os) {
	SunOS {set w 8c}
	Linux {set w 9c}
	Linux64 {set w 9c}
    }
    $canvas create window 0 0 -tags "$dir $dir:list list" \
	-window $frame -width $w -height $h

    # arrange the items in the tree node and change the bitmap
    # to show an "open" directory
    LayoutNode $canvas $tree $dir
    $canvas itemconfig $dir:bitmap -bitmap "@$bitmaps/open_dir.xbm"
    $tree nodeconfig $dir -remove "destroy $frame" 

    # add bindings for resizing the listbox in the canvas
    SetFileListBindings $canvas $tree $frame $frame.list "" $dir
    
}


# remove the list frame for displaying the files in dir

proc RemoveListFrame {canvas tree frame dir} {
    global dirtree bitmaps
    $canvas delete $dir:list
    destroy $frame

    $canvas itemconfig $dir:bitmap -bitmap @$bitmaps/dir.xbm
    $tree nodeconfig $dir -remove "" -border 2
    set dirtree(list) ""
    set dirtree(dir) ""

# Set this node as closed 

    set node [file tail $dir]
   if {![string compare -length 6 $node "action"]} {
       set nn [string last "/" $dir]
       set nodepath [string range $dir 0 [expr $nn -1]]
       set nn [getnode $nodepath]
       global $nn$node
       set [set nn][set node](open) 0
   } {
       global $node
       set [set node](open) 0
   }
}


# event proc called to resize the file listbox in the canvas 
# while the user is dragging its corner

proc ResizeList {canvas tree frame list x y dir {when any}} {
    global dirtree
    
    case "$when" {
	"first" {
	    set dirtree(tlx) $x
	    set dirtree(tly) $y
	} 
	"last" {
	    $tree nodeconfig $dir
	}
	default {
	    set w [$canvas itemcget $dir:list -width]
	    set h [$canvas itemcget $dir:list -height]
	    set nw [expr $w+($x-$dirtree(tlx))]
	    set nh [expr $h+($y-$dirtree(tly))]
	    $canvas itemconfig $dir:list -width $nw -height $nh
	    set dirtree(tlx) $x
	    set dirtree(tly) $y
	}
    }
}


# set the bindings for a file listbox in the tree canvas

proc SetFileListBindings {canvas tree frame list scroll dir} {
    global dirtree
#    bind $frame <ButtonPress-1> "ResizeList $canvas $tree $frame $list %x %y $dir first"
#    bind $frame <Button-Motion> "ResizeList $canvas $tree $frame $list %x %y $dir"
#    bind $frame <ButtonRelease-1> "ResizeList $canvas $tree $frame $list %x %y $dir last"
    
#    bind $list <1>  "set dirtree(list) %W; set dirtree(dir) $dir; [bind Listbox <1>]"
#    bind $list <Double-Button-1> ChooseFile
    
    # set/clear the resize cursor
#    bind $list <Any-Enter> "$frame config -cursor {}"
#    bind $list <Any-Leave> "$frame config -cursor bottom_right_corner"
#    bind $scroll <Any-Enter> "$frame config -cursor {}"
#    bind $scroll <Any-Leave> "$frame config -cursor bottom_right_corner"
}



# This proc is called when the user double-clicks on a filename 
# in one of the listboxes listing the files in a directory.

proc ChooseFile {} {
    global frame dirtree gottext gotsel
    set file [GetFilename]
    if {"$file" == ""} {return}

    set list $dirtree(list)
    set gotsel [$list curselection]

    set i [string first " " $file 0]
    if {$i < 0} return
    set f [string range $file 0 [expr $i -1]]
    set i [string last "/" $f]
    if {$i < 0} return
    set gottext [string range $file [expr $i +1] end]
puts "got $gottext"
}
  

proc PutFilename {f} {
    global dirtree gotsel
    set dir $dirtree(dir)
    set list $dirtree(list)
    if {"$dir" == "" || "$list" == ""} {return ""}
    
    $list delete $gotsel
    $list insert $gotsel $f
    return
}

proc AddFilename {f} {
    global dirtree 
    set dir $dirtree(dir)
    set list $dirtree(list)
    if {"$dir" == "" || "$list" == ""} {return ""}
    
    $list insert end $f
    return
}

proc DelFilename {f} {
    global dirtree gotsel
    set dir $dirtree(dir)
    set list $dirtree(list)
    if {"$dir" == "" || "$list" == ""} {return ""}
    
    $list delete $gotsel
    return
}

# Return the path name of the file currently selected in the
# current node's listbox, or the empty string if none are selected. 

proc GetFilename {} {
    global dirtree
    set dir $dirtree(dir)
    set list $dirtree(list)
    if {"$dir" == "" || "$list" == ""} {return ""}
    
    set sel [$list curselection]
    if {![llength $sel]} {return ""}
    return $dir/[$list get [lindex $sel 0]]
}


# layout the components of the given node depending on whether
# the tree is vertical or horizontal
# (Also take the file listbox into consideration if it is open)

proc LayoutNode {canvas tree dir} {
    set text $dir:text
    set bitmap $dir:bitmap
    set list [$canvas find withtag $dir:list]
    
    if {[$tree cget -layout] == "horizontal"} {
        scan [$canvas bbox $text] "%d %d %d %d" x1 y1 x2 y2
	$canvas itemconfig $bitmap -anchor se
	$canvas coords $bitmap $x1 $y2
	if {"$list" != ""} {
	    scan  [$canvas bbox $text $bitmap] "%d %d %d %d" x1 y1 x2 y2
	    $canvas itemconfig $list -anchor nw
	    $canvas coords $list $x1 $y2
	}
    } else {
        scan [$canvas bbox $bitmap] "%d %d %d %d" x1 y1 x2 y2
	$canvas itemconfig $text -anchor n
	$canvas coords $text [expr "$x1+($x2-$x1)/2"] $y2
	if {"$list" != ""} {
	    scan  [$canvas bbox $text $bitmap] "%d %d %d %d" x1 y1 x2 y2
	    $canvas itemconfig $list -anchor n
	    $canvas coords $list [expr "$x1+($x2-$x1)/2"] $y2
	}
    }
}


proc get-usersetupdir {} {global usersetupdir; return $usersetupdir}

proc call-back {interp proc result} {
   global debug
   if {$debug} then {puts stderr "[self frame-name]: call-back: $interp $proc $result"}
   send $interp $proc [transparent [$result]]
   }
